from ase import Atom, Atoms
from ase.units import Bohr
from ase.parallel import parprint
from gpaw import GPAW
from gpaw.test import equal
from gpaw.lrtddft import LrTDDFT
from gpaw.mpi import world
from gpaw.lrtddft.excited_state import ExcitedState

txt='-'
txt='/dev/null'
io_only=False
#io_only=True
load=False
#load=True
if not io_only:
    R=0.7 # approx. experimental bond length
    a = 3.0
    c = 4.0
    H2 = Atoms([Atom('H', (a / 2, a / 2, (c - R) / 2)),
                Atom('H', (a / 2, a / 2, (c + R) / 2))],
               cell=(a, a, c))
    calc = GPAW(xc='PBE',
                poissonsolver={'name': 'fd'},
                nbands=3, spinpol=False, txt=txt)
    H2.set_calculator(calc)
    H2.get_potential_energy()

    xc='LDA'

    # without spin
    lr = LrTDDFT(calc, xc=xc)
    lr.diagonalize()
    t1 = lr[0]
    lr_calc = lr
    ex = ExcitedState(lr, 0)
    den = ex.get_pseudo_density() * Bohr**3

    # velocity from
    for ozr, ozv in zip(t1.get_oscillator_strength(),
                        t1.get_oscillator_strength('v')):
        equal(ozr, ozv, 0.1)

    # course grids
    for finegrid in [1,0]:
        lr = LrTDDFT(calc, xc=xc, finegrid=finegrid)
        lr.diagonalize()
        t3 = lr[0]
        parprint('finegrid, t1, t3=', finegrid, t1 ,t3)
        equal(t1.get_energy(), t3.get_energy(), 5.e-4)

    # with spin
    
    lr_vspin = LrTDDFT(calc, xc=xc, nspins=2)
    singlet, triplet = lr_vspin.singlets_triplets()
    lr_vspin.diagonalize()
    # the triplet is lower, so that the second is the first singlet
    # excited state
    t2 = lr_vspin[1]
    ex_vspin = ExcitedState(lr_vspin, 1)
    den_vspin = ex_vspin.get_pseudo_density() * Bohr**3

    parprint('with virtual/wo spin t2, t1=',
             t2.get_energy(), t1 .get_energy())
    equal(t1.get_energy(), t2.get_energy(), 5.e-7)
    gd = lr.calculator.density.gd
    finegd = lr.calculator.density.finegd
    ddiff = gd.integrate(abs(den - den_vspin))
    parprint('   density integral, difference=',
             gd.integrate(den), gd.integrate(den_vspin), ddiff)
    parprint('   aed density integral',
             finegd.integrate(ex.get_all_electron_density() * Bohr**3),
             finegd.integrate(ex_vspin.get_all_electron_density() * Bohr**3))
    assert(ddiff < 1.e-4)

    if not load:
        c_spin = GPAW(xc='PBE', nbands=2,
                      poissonsolver={'name': 'fd'},
                      spinpol=True, parallel={'domain': world.size},
                      txt=txt)
        H2.set_calculator(c_spin)
        c_spin.calculate(H2)
##        c_spin.write('H2spin.gpw', 'all')
    else:
        c_spin = GPAW('H2spin.gpw', txt=txt)
    lr_spin = LrTDDFT(c_spin, xc=xc)
    lr_spin.diagonalize()
    for i in range(2):
        parprint('i, real, virtual spin: ', i, lr_vspin[i], lr_spin[i])
        equal(lr_vspin[i].get_energy(), lr_spin[i].get_energy(), 6.e-6)
        ex_vspin = ExcitedState(lr_vspin, i)
        den_vspin = ex_vspin.get_pseudo_density() * Bohr**3
        ex_spin = ExcitedState(lr_spin, i)
        den_spin = ex_spin.get_pseudo_density() * Bohr**3
        ddiff = gd.integrate(abs(den_vspin - den_spin))
        parprint('   density integral, difference=',
                 gd.integrate(den_vspin), gd.integrate(den_spin),
                 ddiff)
        parprint('   aed density integral',
                 finegd.integrate(ex_vspin.get_all_electron_density() 
                                  * Bohr**3),
                 finegd.integrate(ex_spin.get_all_electron_density() 
                                  * Bohr**3))
        assert(ddiff < 3.e-3), ddiff

    # singlet/triplet separation
    precision = 1.e-5
    singlet.diagonalize()
    equal(singlet[0].get_energy(), lr_spin[1].get_energy(), precision)
    equal(singlet[0].get_oscillator_strength()[0],
          lr_spin[1].get_oscillator_strength()[0], precision)
    triplet.diagonalize()
    equal(triplet[0].get_oscillator_strength()[0], 0)
    equal(triplet[0].get_energy(), lr_spin[0].get_energy(), precision)
    equal(triplet[0].get_oscillator_strength()[0], 0)

# io
fname = 'lr.dat.gz'
if not io_only:
    lr.write(fname)
    world.barrier()
lr = LrTDDFT(fname)
lr.diagonalize()
t4 = lr[0]

if not io_only:
    equal(t3.get_energy(), t4.get_energy(), 1e-6)

e4 = t4.get_energy()
equal(e4, 0.675814, 2e-4)

# excited state with forces
if 1:
    accuracy = 0.3
    exst = ExcitedState(lr_calc, 0)
    forces = exst.get_forces(H2)
    parprint('forces=', forces)
    for c in range(2):
        equal(forces[0,c], 0.0, accuracy)
        equal(forces[1,c], 0.0, accuracy)
    equal(forces[0, 2] + forces[1, 2], 0.0, accuracy)



from ase import Atoms
import numpy as np
from gpaw import GPAW, FermiDirac, Davidson, Mixer
from gpaw.cdft.cdft import CDFT
from gpaw.cdft.cdft_coupling import CouplingParameters
from gpaw.mpi import size

distance = 2.5
sys = Atoms('He2', positions = ([0.,0.,0.],[0.,0.,distance]))
sys.center(3)
sys.set_pbc(False)
sys.set_initial_magnetic_moments([0.5,0.5])

calc_b = GPAW(h = 0.2,
              mode='fd',
              basis='dzp',
              charge=1,
              xc='PBE',symmetry='off',
              occupations = FermiDirac(0., fixmagmom = True),
              eigensolver = Davidson(3),
              spinpol = True,
              nbands = 4,
              mixer = Mixer(beta=0.25, nmaxold=3, weight=100.0),
              txt='He2+_final_%3.2f.txt' % distance,
              convergence={'eigenstates':1.0e-4,'density':1.0e-1, 'energy':1e-1,'bands':4})


cdft_b = CDFT(calc = calc_b,
              atoms=sys,
              charge_regions = [[1]],
              charges = [1],
              charge_coefs = [2.7],
              method = 'L-BFGS-B',
              txt = 'He2+_final_%3.2f.cdft' % distance,
              minimizer_options={'gtol':0.01})
sys.set_calculator(cdft_b)
sys.get_potential_energy()

if size == 1:
    coupling = CouplingParameters(cdft_b, cdft_b, AE = False)
    overlaps = coupling.get_pair_density_matrix(cdft_b.calc, cdft_b.calc)[0]
    print(overlaps)

    for i in [0,1,2]:
        assert (np.isclose(np.real(overlaps[0,i,i]),1.))



from ase import Atoms
from gpaw import GPAW, PW
from gpaw.response.df import DielectricFunction
from gpaw.test import equal, findpeak

# Comparing the EELS spectrum of sodium for different block 
# parallelizations. Intended to be run with 8 cores.

a = 4.23 / 2.0
a1 = Atoms('Na',
           scaled_positions=[[0, 0, 0]],
           cell=(a, a, a),
           pbc=True)

a1.calc = GPAW(gpts=(10, 10, 10),
               mode=PW(300),
               kpts={'size': (30, 30, 30), 'gamma': True},
               parallel={'band': 1},
               txt='small.txt')

a1.get_potential_energy()
a1.calc.diagonalize_full_hamiltonian(nbands=20)
a1.calc.write('gs_Na.gpw', 'all')

kwargs = {'integrationmode': 'tetrahedron integration',
          'ecut': 400}

# Calculate the dielectric functions
df1 = DielectricFunction('gs_Na.gpw',
                         nblocks=1,
                         txt='1block.txt',
                         **kwargs)

df1NLFCx, df1LFCx = df1.get_dielectric_function(direction='x')
df1NLFCy, df1LFCy = df1.get_dielectric_function(direction='y')
df1NLFCz, df1LFCz = df1.get_dielectric_function(direction='z')

df2 = DielectricFunction('gs_Na.gpw',
                         nblocks=4,
                         txt='4block.txt',
                         **kwargs)

df2NLFCx, df2LFCx = df2.get_dielectric_function(direction='x')
df2NLFCy, df2LFCy = df2.get_dielectric_function(direction='y')
df2NLFCz, df2LFCz = df2.get_dielectric_function(direction='z')

# Compare plasmon frequencies and intensities
w_w = df1.chi0.omega_w

w1, I1 = findpeak(w_w, -(1. / df1LFCx).imag)
w2, I2 = findpeak(w_w, -(1. / df2LFCx).imag)
equal(w1, w2, 1e-2)
equal(I1, I2, 1e-3)



from unittest import SkipTest
from ase import Atoms
from gpaw import GPAW
from gpaw.mpi import size

if size > 2:
    raise SkipTest()

a = Atoms('OO', [[0, 0, 0], [0, 0, 1.1]], magmoms=[1, 1], pbc=(1, 0, 0))
a.center(vacuum=2.5)
a.calc = GPAW(mode='pw',
              kpts=(2, 1, 1))
f0 = a.get_forces()

a.calc = GPAW(mode='pw',
              kpts=(2, 1, 1),
              symmetry='off',
              experimental={'magmoms': [[0, 0.5, 0.5], [0, 0, 1]]})
f = a.get_forces()

assert abs(f - f0).max() < 0.01

a.calc.write('o2.gpw')
a.calc.write('o2w.gpw', 'all')
GPAW('o2w.gpw')



"""Test of HGH pseudopotential implementation.

This is the canonical makes-sure-nothing-breaks test, which checks that the
numbers do not change from whatever they were before.

The test runs an HGH calculation on a misconfigured H2O molecule, such that
the forces are nonzero.

Energy is compared to a previous calculation; if it differs significantly,
that is considered an error.

Forces are compared to a previous finite-difference result.
"""

from __future__ import print_function

import numpy as np
from ase.build import molecule

from gpaw import GPAW, PoissonSolver
from gpaw.test import equal

mol = molecule('H2O')
mol.rattle(0.2)
mol.center(vacuum=2.0)
calc = GPAW(nbands=6,
            poissonsolver=PoissonSolver('fd'),
            gpts=(32, 40, 40),
            setups='hgh',
            convergence=dict(eigenstates=1e-9, density=1e-5, energy=0.3e-5),
            txt='-')
mol.set_calculator(calc)
e = mol.get_potential_energy()
niter = calc.get_number_of_iterations()
F_ac = mol.get_forces()

F_ac_ref = np.array([[ 7.33077718,  3.81069249, -6.07405156],
                     [-0.9079617 , -1.18203514,  3.43777589],
                     [-0.61642527, -0.41889306,  2.332415  ]])

eref = -468.527121304

eerr = abs(e - eref)

print('energy', e)
print('ref energy', eref)
print('energy error', eerr)

print('forces')
print(F_ac)

print('ref forces')
print(F_ac_ref)

ferr = np.abs(F_ac - F_ac_ref).max()
print('max force error', ferr)

fdcheck = False

if fdcheck:
    from ase.calculators.test import numeric_forces
    F_ac_fd = numeric_forces(mol)

    print('Self-consistent forces')
    print(F_ac)
    print('FD forces')
    print(F_ac_fd)
    print()
    print(repr(F_ac_fd))
    print()
    err = np.abs(F_ac - F_ac_fd).max()
    print('max err', err)

wfs = calc.wfs
gd = wfs.gd
psit_nG = wfs.kpt_u[0].psit_nG
dH_asp = calc.hamiltonian.dH_asp

assert eerr < 1e-3, 'energy changed from reference'
assert ferr < 0.015, 'forces do not match FD check'

# Sanity check.  In HGH, the atomic Hamiltonian is constant.
# Also the projectors should be normalized
for a, dH_sp in dH_asp.items():
    dH_p = dH_sp[0]
    K_p = wfs.setups[a].K_p
    #B_ii = wfs.setups[a].B_ii
    #assert np.abs(B_ii.diagonal() - 1).max() < 1e-3
    #print 'B_ii'
    #print wfs.setups[a].B_ii
    # Actually, H2O might not be such a good test, since there'll only
    # be one element in the atomic Hamiltonian for O and zero for H.
    #print 'dH_p', dH_p
    #print 'K_p', K_p

    assert np.abs(dH_p - K_p).max() < 1e-10, 'atomic Hamiltonian changed'

    #h_ii = setup.data.h_ii
    #print 'h_ii', h_ii
    #print 'dH_ii', dH_ii

# Sanity check: HGH is normconserving
for psit_G in psit_nG:
    norm = gd.integrate(psit_G**2) # Around 1e-15 !  Surprisingly good.
    assert abs(1 - norm) < 1e-10, 'Not normconserving'

energy_tolerance = 0.0003
equal(e, eref, energy_tolerance)



import numpy as np
from ase.build import bulk
from gpaw import GPAW
from ase.dft.kpoints import monkhorst_pack


for kpt in (3, 4):
    kpts = (kpt, kpt, kpt)
    bzk_kc = monkhorst_pack(kpts)
    shift_c = []
    for Nk in kpts:
        if Nk % 2 == 0:
            shift_c.append(0.5 / Nk)
        else:
            shift_c.append(0.)
    bzk_kc += shift_c

    atoms = bulk('Si', 'diamond', a=5.431)
    calc = GPAW(h=0.2,
                kpts=bzk_kc)
    
    atoms.set_calculator(calc)
    atoms.get_potential_energy()

    kd = calc.wfs.kd
    bzq_qc = kd.get_bz_q_points()
    ibzq_qc = kd.get_ibz_q_points(bzq_qc, calc.wfs.kd.symmetry.op_scc)[0]

    assert np.abs(bzq_qc - kd.bzk_kc).sum() < 1e-8
    assert np.abs(ibzq_qc - kd.ibzk_kc).sum() < 1e-8



from __future__ import print_function

import os

from ase.build import bulk
import numpy as np

from gpaw import GPAW
from gpaw import restart
from _gpaw import symmetrize
from gpaw.symmetry import Symmetry
from gpaw.utilities.tools import md5_array

# Normal ground state calculation
atoms = bulk('Si')
calc = GPAW(h=0.24, kpts=(4,4,4),
            convergence={'eigenstates' : 1e-4, 'density' : 1e-3})
atoms.set_calculator(calc)
atoms.get_potential_energy()

# "Bandstructure" calculation (only Gamma point here)
kpts = np.array(((0, 0, 0),))
calc.set(fixdensity=True, kpts=kpts)
atoms.get_potential_energy()
calc.write('Si_gamma.gpw', mode='all')

# Analyse symmetries of wave functions
atoms, calc = restart('Si_gamma.gpw', txt=None)

# Find symmetries (in Gamma-point calculations calculator does not
# use symmetry)
sym = Symmetry(calc.wfs.setups.id_a, atoms.cell, atoms.pbc)
sym.analyze(atoms.get_scaled_positions())

def find_classes(op_all_scc):
    # Find classes of group represented by matrices op_all_scc
    # and return representative operations
    op_scc = [op_all_scc[0]]
    for op1 in op_all_scc[1:]:
        new_class = True
        for op2 in op_all_scc:
            op_tmp = (np.dot(np.dot(op2, op1), np.linalg.inv(op2).astype(int)))
            # Check whether operation op1 belongs to existing class
            for op in op_scc:
                if np.all((op_tmp - op) == 0):
                    new_class = False
                    break
            if not new_class:
                break
        if new_class:
            op_scc.append(op1)
    return op_scc

op_scc = find_classes(sym.op_scc)
nops = len(op_scc)
nbands = calc.wfs.bd.nbands

characters = np.zeros((nbands, nops))

degeneracy_threshold = 1e-2 # Threshold for considering bands degenerate
symdone = np.zeros(nbands, bool)

eig_n = calc.get_eigenvalues()
for n in range(nbands):
    # For degenerate states analysis might be already done
    if symdone[n]:
        continue
    # Find degenerate bands
    ndeg = 0
    m = n
    while abs(eig_n[m] - eig_n[n]) < degeneracy_threshold:
        ndeg += 1
        m += 1
        if m == nbands:
            break

    # Representation matrix
    representation_nn = np.zeros((ndeg, ndeg, nops))
    for nop, op_cc in enumerate(op_scc):
        # Bands from n to m are (possibly) degenerate
        for n1 in range(n, m):
            for n2 in range(n, m):
                wf1 = calc.get_pseudo_wave_function(band=n1)
                wf2 = calc.get_pseudo_wave_function(band=n2)
                norm1 = np.sqrt(calc.wfs.gd.integrate(wf1, wf1))
                norm2 = np.sqrt(calc.wfs.gd.integrate(wf2, wf2).real)
                wf_rot = np.zeros_like(wf2)
                symmetrize(wf2, wf_rot, op_cc)
                # Indices of representation matrix are from 0 to ndeg
                i1, i2 = n1 - n, n2 - n
                representation_nn[i1, i2, nop] = calc.wfs.gd.integrate(wf1,
                                                                       wf_rot)
                representation_nn[i1, i2, nop] /= norm1 * norm2

    # Calculate traces of irreducible representations
    # If bands i1 and i2 are accidentally degenerate (i.e. not due to symmetry)
    # they belong to different irreducible representations and the
    # corresponding representation matrix elements are zero  for all
    # symmetry operations.
    for i1 in range(ndeg):
        for i2 in range(ndeg):
            if np.any(abs(representation_nn[i1, i2, :]) > 0.01):
                characters[n + i1, :] += representation_nn[i2, i2, :]
        symdone[n + i1] = True

# Use four decimals for characters
characters = np.round(characters, 4)
# Use characters as fingerprints
fingerprints = np.array([md5_array(row) for row in characters])

fmt = "%6.4f " * nops
for i in range(nbands):
    print(fmt % tuple([c for c in characters[i, :nops]]))

# Correct?!? character table
characters_reference = np.array(((1.0, 1.0, 1.0, 1.0, 1.0),
                                 (3.0, 1.0, 0.0, -1.0, -1.0),
                                 (3.0, 1.0, 0.0, -1.0, -1.0),
                                 (3.0, 1.0, 0.0, -1.0, -1.0),
                                 (3.0, 1.0, 0.0, -1.0, -1.0),
                                 (3.0, 1.0, 0.0, -1.0, -1.0),
                                 (3.0, 1.0, 0.0, -1.0, -1.0),
                                 (1.0, 1.0, 1.0, 1.0, 1.0)))
assert np.all(np.abs(characters_reference-characters) < 1.0e-4)
os.remove('Si_gamma.gpw')



"""
Calculate optical transition strengths.
"""

# General modules
import numpy as np

# Script modules
from ase.build import bulk

from gpaw import GPAW, PW, FermiDirac
from gpaw.response.tool import (get_bz_transitions,
                                get_chi0_integrand,
                                get_degeneracy_matrix,
                                get_individual_transition_strengths)
from gpaw.test import equal

# ------------------- Inputs ------------------- #

# Part 1: ground state calculation
pw = 200
kpts = 3
nbands = 8

# Part 2: optical transitions calculation
response = 'density'
spins = 'all'
q_c = [0., 0., 0.]
bzk_kc = np.array([[0., 0., 0.]])

# ------------------- Script ------------------- #

# Part 1: ground state calculation
a = 5.431
atoms = bulk('Si', 'diamond', a=a)

calc = GPAW(mode=PW(pw),
            kpts=(kpts, kpts, kpts),
            nbands=nbands,
            convergence={'bands': -1},
            xc='LDA',
            occupations=FermiDirac(0.001))  # use small FD smearing

atoms.set_calculator(calc)
atoms.get_potential_energy()  # get ground state density

calc.write('si.gpw', 'all')  # write wavefunctions

# Part 2: optical transition calculation

pair, pd, domainarg_td = get_bz_transitions('si.gpw', q_c, bzk_kc,
                                            response=response, spins=spins,
                                            ecut=10)

# non-empty bands
n_n = np.arange(0, pair.nocc2)
# not completely filled bands
m_m = np.arange(pair.nocc1, pair.calc.wfs.bd.nbands)

nt = len(domainarg_td)
nn = len(n_n)
nm = len(m_m)
nG = pd.ngmax
optical_limit = np.allclose(q_c, 0.) and response == 'density'

n_tnmG = np.zeros((nt, nn, nm, nG + 2 * optical_limit), dtype=complex)
df_tnm = np.zeros((nt, nn, nm), dtype=float)
eps_tn = np.zeros((nt, nn), dtype=float)
eps_tm = np.zeros((nt, nm), dtype=float)
for t, domainarg_d in enumerate(domainarg_td):
    (n_tnmG[t], df_tnm[t],
     eps_tn[t], eps_tm[t]) = get_chi0_integrand(pair, pd,
                                                n_n, m_m,
                                                *domainarg_d)

testNM_ibN = np.array([[[0], [4, 5, 6]], [[0], [7]],
                       [[1, 2, 3], [4, 5, 6]], [[1, 2, 3], [7]]])
testts_iG = np.array([[0.07, 0.07, 0.07], [0.00, 0.00, 0.00],
                      [51.34, 51.34, 51.34], [22.69, 22.69, 22.69]])

for t, (domainarg_d, n_nmG,
        df_nm, eps_n, eps_m) in enumerate(zip(domainarg_td, n_tnmG, df_tnm,
                                              eps_tn, eps_tm)):

    # Find degeneracies
    degmat_Nn, eps_N = get_degeneracy_matrix(eps_n, tol=1.e-2)
    degmat_Mm, eps_M = get_degeneracy_matrix(eps_m, tol=1.e-2)

    # Find diagonal transition strengths
    its_nmG = np.zeros((nn, nm, 1 + 2 * optical_limit))
    for G in range(1 + 2 * optical_limit):
        its_nmG[:, :, G] = get_individual_transition_strengths(n_nmG, df_nm,
                                                               G, G)

    # Find unique transition strengths
    its_NmG = np.tensordot(degmat_Nn, its_nmG, axes=(1, 0))
    ts_MNG = np.tensordot(degmat_Mm, its_NmG, axes=(1, 1))
    ts_NMG = np.transpose(ts_MNG, (1, 0, 2))

    i = 0
    for N, ts_MG in enumerate(ts_NMG):
        for M, ts_G in enumerate(ts_MG):
            degN_n = n_n[np.where(degmat_Nn[N])]
            degM_m = m_m[np.where(degmat_Mm[M])]

            for testn, n in zip(testNM_ibN[i, 0], degN_n):
                equal(testn, n, 0.5)
            for testm, m in zip(testNM_ibN[i, 1], degM_m):
                equal(testm, m, 0.5)

            for testts, ts in zip(testts_iG[i], ts_G):
                print(ts)
                equal(testts, ts, 0.01)

            i += 1



import numpy as np
from ase import Atoms
from ase.neighborlist import NeighborList

from gpaw.lcao.overlap import AtomicDisplacement, TwoCenterIntegralCalculator


class DistsAndOffsets:
    def __init__(self, nl, spos_ac, cell_cv):
        #assert nl.bothways
        r_and_offset_aao = {}

        def add(a1, a2, R_c, offset):
            r_and_offset_aao.setdefault((a1, a2), []).append((R_c, offset))

        for a1, spos1_c in enumerate(spos_ac):
            a2_a, offsets = nl.get_neighbors(a1)
            for a2, offset in zip(a2_a, offsets):
                spos2_c = spos_ac[a2] + offset

                R_c = np.dot(spos2_c - spos1_c, cell_cv)
                add(a1, a2, R_c, offset)
                if a1 != a2 or offset.any():
                    add(a2, a1, -R_c, -offset)
        self.r_and_offset_aao = r_and_offset_aao

    def get(self, a1, a2):
        R_ca_and_offset_a = self.r_and_offset_aao.get((a1, a2))
        return R_ca_and_offset_a


def newoverlap(wfs, spos_ac):
    assert wfs.ksl.block_comm.size == wfs.gd.comm.size * wfs.bd.comm.size
    #even_part = EvenPartitioning(wfs.gd.comm, #wfs.ksl.block_comm,
    #                             len(wfs.atom_partition.rank_a))
    #atom_partition = even_part.as_atom_partition()
    # XXXXXXXXXXXXXXX
    atom_partition = wfs.atom_partition
    
    tci = wfs.tci

    gd = wfs.gd
    kd = wfs.kd
    nq = len(kd.ibzk_qc)

    # New neighbor list because we want it "both ways", heh.  Or do we?
    neighbors = NeighborList(tci.cutoff_a, skin=0,
                             sorted=True, self_interaction=True,
                             bothways=False)
    atoms = Atoms('X%d' % len(tci.cutoff_a), cell=gd.cell_cv, pbc=gd.pbc_c)
    atoms.set_scaled_positions(spos_ac)
    neighbors.update(atoms)
    
    # XXX
    pcutoff_a = []
    phicutoff_a = []
    for setup in wfs.setups:
        if setup.pt_j:
            pcutoff = max([pt.get_cutoff() for pt in setup.pt_j])
        else:
            pcutoff = 0.0
        if setup.phit_j:
            phicutoff = max([phit.get_cutoff() for phit in setup.phit_j])
        else:
            phicutoff = 0.0
        pcutoff_a.append(pcutoff)
        phicutoff_a.append(phicutoff)

    # Calculate the projector--basis function overlaps:
    #
    #    a1        ~a1
    #   P      = < p   | Phi   > ,
    #    i mu       i       mu
    #
    # i.e. projector is on a1 and basis function is on what we will call a2.

    overlapcalc = TwoCenterIntegralCalculator(wfs.kd.ibzk_qc,
                                              derivative=False)
    
    P_aaqim = {}  # keys: (a1, a2).  Values: matrix blocks
    dists_and_offsets = DistsAndOffsets(neighbors, spos_ac, gd.cell_cv)

    #ng = 2**extra_parameters.get('log2ng', 10)
    #transformer = FourierTransformer(rcmax, ng)
    #tsoc = TwoSiteOverlapCalculator(transformer)
    #msoc = ManySiteOverlapCalculator(tsoc, I_a, I_a)

    msoc = wfs.tci.msoc

    phit_Ij = [setup.phit_j for setup in tci.setups_I]
    l_Ij = []
    for phit_j in phit_Ij:
        l_Ij.append([phit.get_angular_momentum_number()
                     for phit in phit_j])

    pt_l_Ij = [setup.l_j for setup in tci.setups_I]
    pt_Ij = [setup.pt_j for setup in tci.setups_I]
    phit_Ijq = msoc.transform(phit_Ij)
    pt_Ijq = msoc.transform(pt_Ij)

    #self.Theta_expansions = msoc.calculate_expansions(l_Ij, phit_Ijq,
    #                                                  l_Ij, phit_Ijq)
    #self.T_expansions = msoc.calculate_kinetic_expansions(l_Ij, phit_Ijq)
    P_expansions = msoc.calculate_expansions(pt_l_Ij, pt_Ijq,
                                             l_Ij, phit_Ijq)
    P_neighbors_a = {}
    
    for a1 in atom_partition.my_indices:
        for a2 in range(len(wfs.setups)):
            R_ca_and_offset_a = dists_and_offsets.get(a1, a2)
            if R_ca_and_offset_a is None:  # No overlap between a1 and a2
                continue

            maxdistance = pcutoff_a[a1] + phicutoff_a[a2]
            expansion = P_expansions.get(a1, a2)
            P_qim = expansion.zeros((nq,), dtype=wfs.dtype)
            disp = None
            for R_c, offset in R_ca_and_offset_a:
                r = np.linalg.norm(R_c)
                if r > maxdistance:
                    continue
                
                # Below lines are meant to make use of symmetry.  Will not
                # be relevant for P.
                #remainder = (a1 + a2) % 2
                #if a1 < a2 and not remainder:
                #    continue
                # if a1 > a2 and remainder:
                #    continue
                
                phases = overlapcalc.phaseclass(overlapcalc.ibzk_qc, offset)
                disp = AtomicDisplacement(None, a1, a2, R_c, offset, phases)
                disp.evaluate_overlap(expansion, P_qim)
            
            if disp is not None: # there was at least one non-zero overlap
                assert (a1, a2) not in P_aaqim
                P_aaqim[(a1, a2)] = P_qim
                P_neighbors_a.setdefault(a1, []).append(a2)
    
    return P_neighbors_a, P_aaqim



from itertools import product

import numpy as np

from gpaw.response.integrators import TetrahedronIntegrator
from gpaw.response.chi0 import ArrayDescriptor
from gpaw.test import equal


def unit(x_c):
    return np.array([[1.]], complex)


def unit_sphere(x_c):
    return np.array([(x_c**2).sum()**0.5], float)

# Test tetrahedron integrator
cell_cv = np.eye(3)
integrator = TetrahedronIntegrator(cell_cv)
x_g = np.linspace(-1, 1, 30)
x_gc = np.array([comb for comb in product(*([x_g] * 3))])

domain = (x_gc,)
out_wxx = np.zeros((1, 1, 1), complex)
integrator.integrate(kind='spectral function',
                     domain=domain,
                     integrand=(unit, unit_sphere),
                     x=ArrayDescriptor([-1.0]),
                     out_wxx=out_wxx)

equal(out_wxx[0, 0, 0], 4 * np.pi, 1e-2,
      msg='Integrated area of unit sphere is not 4 * pi')



from ase import Atoms, Atom
from ase.vibrations.resonant_raman import ResonantRaman
from ase.vibrations.albrecht import Albrecht
from ase.parallel import world, DummyMPI

from gpaw import GPAW
from gpaw.lrtddft.kssingle import KSSingles
from gpaw.analyse.overlap import Overlap
from gpaw.test import equal

txt = '-'
txt = None
load = True
load = False
xc = 'LDA'

R = 0.7  # approx. experimental bond length
a = 4.0
c = 5.0
H2 = Atoms([Atom('H', (a / 2, a / 2, (c - R) / 2)),
            Atom('H', (a / 2, a / 2, (c + R) / 2))],
           cell=(a, a, c))

calc = GPAW(xc=xc, nbands=3, spinpol=False, eigensolver='rmm-diis', txt=txt)
H2.set_calculator(calc)
H2.get_potential_energy()

gsname = exname = 'rraman'
exkwargs={'eps':0.0, 'jend':1}
pz = ResonantRaman(H2, KSSingles, gsname=gsname, exname=exname,
                    exkwargs=exkwargs,
                   overlap=lambda x, y: Overlap(x).pseudo(y))
pz.run()

# check size
kss = KSSingles('rraman-d0.010.eq.ex.gz')
assert(len(kss) == 1)

om = 5

# Does Albrecht A work at all ?
# -----------------------------

al = Albrecht(H2, KSSingles, gsname=gsname, exname=exname,
              verbose=True, overlap=True)
ai = al.absolute_intensity(omega=om)[-1]
i = al.intensity(omega=om)[-1]

# parallel

if world.size > 1 and world.rank == 0:
    # single core
    comm = DummyMPI()
    pzsi = Albrecht(H2, KSSingles, gsname=gsname, exname=exname,
                   comm=comm, overlap=True, verbose=True)
    isi = pzsi.intensity(omega=om)[-1]
    equal(isi, i, 1e-11)

# Compare singles and multiples in Albrecht A
# -------------------------------------------

alas = Albrecht(H2, KSSingles, gsname=gsname, exname=exname,
              approximation='Albrecht A')
ints = alas.intensity(omega=om)[-1]
alam = Albrecht(H2, KSSingles, gsname=gsname, exname=exname,
                approximation='Albrecht A',
                skip=5, combinations=2)
# single excitation energies should agree
equal(ints, alam.intensity(omega=om)[0] , 1e-11)
alam.summary(omega=5, method='frederiksen')




from __future__ import print_function
from ase.build import molecule

from gpaw import GPAW, FermiDirac, PoissonSolver
from gpaw.test import equal, gen

# Generate setup for oxygen with a core-hole:
def xc(name):
    return {'name': name, 'stencil': 1}


gen('O', name='fch1s', xcname='PBE', corehole=(1, 0, 1.0))

atoms = molecule('H2O')
atoms.center(vacuum=2.5)

calc = GPAW(xc=xc('PBE'),
            poissonsolver=PoissonSolver('fd',
                                        use_charge_center=True))
atoms.set_calculator(calc)
e1 = atoms.get_potential_energy() + calc.get_reference_energy()
niter1 = calc.get_number_of_iterations()

atoms[0].magmom = 1
calc.set(charge=-1,
         setups={'O': 'fch1s'},
         occupations=FermiDirac(0.0, fixmagmom=True))
e2 = atoms.get_potential_energy() + calc.get_reference_energy()
niter2 = calc.get_number_of_iterations()

atoms[0].magmom = 0
calc.set(charge=0,
         setups={'O': 'fch1s'},
         occupations=FermiDirac(0.0, fixmagmom=True),
         spinpol=True)
e3 = atoms.get_potential_energy() + calc.get_reference_energy()
niter3 = calc.get_number_of_iterations()

print('Energy difference %.3f eV' % (e2 - e1))
print('XPS %.3f eV' % (e3 - e1))

print(e2 - e1)
print(e3 - e1)
assert abs(e2 - e1 - 533.070) < 0.001
assert abs(e3 - e1 - 538.549) < 0.001

energy_tolerance = 0.02
niter_tolerance = 1
print(e1, niter1)
print(e2, niter2)
print(e3, niter3)
equal(e1, -2080.3651, energy_tolerance)
equal(e2, -1547.2944, energy_tolerance)
equal(e3, -1541.8152, energy_tolerance)



from __future__ import print_function
from ase import Atom, Atoms
from gpaw import GPAW, Davidson, Mixer
from gpaw.test import equal
from gpaw.xc.hybrid import HybridXC

def xc(name):
    return {'name': name, 'stencil': 1}

a = 6.  # Size of unit cell (Angstrom)
c = a / 2
# Hydrogen atom:
atom = Atoms([Atom('H', (c, c, c), magmom=1)],
             cell=(a, a, a), pbc=False)

# gpaw calculator:
calc = GPAW(gpts=(32, 32, 32), nbands=1, xc=xc('PBE'), txt='H.txt',
            eigensolver=Davidson(12),
            mixer=Mixer(0.5, 5),
            parallel=dict(kpt=1),
            convergence=dict(eigenstates=3.3e-8))
atom.set_calculator(calc)

e1 = atom.get_potential_energy()
niter1 = calc.get_number_of_iterations()
print('start')
de1t = calc.get_xc_difference(xc('TPSS'))
de1m = calc.get_xc_difference(xc('M06-L'))
de1x = calc.get_xc_difference(HybridXC('EXX', stencil=1, finegrid=True))
de1xb = calc.get_xc_difference(HybridXC('EXX', stencil=1, finegrid=False))
print('stop')

# Hydrogen molecule:
d = 0.74  # Experimental bond length
molecule = Atoms([Atom('H', (c - d / 2, c, c)),
                  Atom('H', (c + d / 2, c, c))],
                 cell=(a, a, a), pbc=False)

calc.set(txt='H2.txt')
molecule.set_calculator(calc)
e2 = molecule.get_potential_energy()
niter2 = calc.get_number_of_iterations()
de2t = calc.get_xc_difference(xc('TPSS'))
de2m = calc.get_xc_difference(xc('M06-L'))
de2x = calc.get_xc_difference(HybridXC('EXX', stencil=1, finegrid=True))
de2xb = calc.get_xc_difference(HybridXC('EXX', stencil=1, finegrid=False))

print('hydrogen atom energy:     %5.2f eV' % e1)
print('hydrogen molecule energy: %5.2f eV' % e2)
print('atomization energy:       %5.2f eV' % (2 * e1 - e2))
print('atomization energy  TPSS: %5.2f eV' % (2 * (e1 + de1t) - (e2 + de2t)))
print('atomization energy  M06-L: %5.2f eV' % (2 * (e1 + de1m) - (e2 + de2m)))
print('atomization energy   EXX: %5.2f eV' % (2 * (e1 + de1x) - (e2 + de2x)))
print('atomization energy   EXX: %5.2f eV' % (2 * (e1 + de1xb) - (e2 + de2xb)))
PBETPSSdifference = (2 * e1 - e2) - (2 * (e1 + de1t) - (e2 + de2t))
PBEM06Ldifference = (2 * e1 - e2) - (2 * (e1 + de1m) - (e2 + de2m))
PBEEXXdifference = (2 * e1 - e2) - (2 * (e1 + de1x) - (e2 + de2x))
PBEEXXbdifference = (2 * e1 - e2) - (2 * (e1 + de1xb) - (e2 + de2xb))
print(PBETPSSdifference)
print(PBEM06Ldifference)
print(PBEEXXdifference)
print(PBEEXXbdifference)
# TPSS value is from JCP 120 (15) 6898, 2004
# e.g. Table VII: DE(PBE - TPSS) = (104.6-112.9)*kcal/mol
# EXX value is from PRL 77, 3865 (1996)
equal(PBETPSSdifference, -0.3599, 0.04)
equal(PBEM06Ldifference, -0.169, 0.01)
equal(PBEEXXdifference, 0.91, 0.005)
equal(PBEEXXbdifference, 0.91, 0.005)

energy_tolerance = 0.002
niter_tolerance = 0
equal(e1, -1.081638, energy_tolerance)
equal(e2, -6.726356, energy_tolerance)



import numpy as np
from ase import Atoms
from gpaw.fdtd.poisson_fdtd import QSFDTD
from gpaw.fdtd.polarizable_material import PermittivityPlus, PolarizableMaterial, PolarizableSphere
from gpaw.mpi import world
from gpaw.test import equal

# This test does the same calculation as ed.py, but using QSFDTD wrapper instead

# Accuracy
energy_eps = 0.0005

# Whole simulation cell (Angstroms)
cell = [20, 20, 30]

# Quantum subsystem
atom_center = np.array([10.0, 10.0, 20.0])
atoms = Atoms('Na2', [atom_center + [0.0, 0.0, -1.50],
                      atom_center + [0.0, 0.0, +1.50]])

# Classical subsystem
sphere_center = np.array([10.0, 10.0, 10.0])
classical_material = PolarizableMaterial()
classical_material.add_component(PolarizableSphere(permittivity = PermittivityPlus(data = [[1.20, 0.20, 25.0]]),
                                                   center = sphere_center,
                                                   radius = 5.0
                                                   ))

# Wrap calculators
qsfdtd = QSFDTD(classical_material = classical_material,
                atoms              = atoms,
                cells              = (cell, 2.50),
                spacings           = [1.60, 0.40],
                remove_moments     = (1, 4),
                communicator       = world)

# Run
qsfdtd.ground_state('gs.gpw', eigensolver='cg', nbands=-1, convergence={'energy': energy_eps}, experimental={'niter_fixdensity': 2})
equal(qsfdtd.energy, -0.631881, energy_eps * qsfdtd.gs_calc.get_number_of_electrons())
qsfdtd.time_propagation('gs.gpw', kick_strength=[0.000, 0.000, 0.001], time_step=10, iterations=5, dipole_moment_file='dm.dat', restart_file='td.gpw')
qsfdtd.time_propagation('td.gpw', kick_strength=None, time_step=10, iterations=5, dipole_moment_file='dm.dat')

# Test
ref_cl_dipole_moment = [  5.25374117e-14,  5.75811267e-14,  3.08349334e-02]
ref_qm_dipole_moment = [  1.78620337e-11, -1.57782578e-11,  5.21368300e-01]
#print("ref_cl_dipole_moment = %s" % qsfdtd.td_calc.hamiltonian.poisson.get_classical_dipole_moment())
#print("ref_qm_dipole_moment = %s" % qsfdtd.td_calc.hamiltonian.poisson.get_quantum_dipole_moment())

tol = 1e-4
equal(qsfdtd.td_calc.hamiltonian.poisson.get_classical_dipole_moment(), ref_cl_dipole_moment, tol)
equal(qsfdtd.td_calc.hamiltonian.poisson.get_quantum_dipole_moment(), ref_qm_dipole_moment, tol)



from __future__ import print_function

# Copyright (C) 2003  CAMP
# Please see the accompanying LICENSE file for further information.

import sys
import time
import math

import numpy as np
from ase.utils.timing import Timer

import gpaw.mpi as mpi


class NullTimer:
    """Compatible with Timer and StepTimer interfaces.  Does nothing."""
    def __init__(self):
        pass

    def print_info(self, calc):
        pass

    def start(self, name):
        pass

    def stop(self, name=None):
        pass

    def get_time(self, name):
        return 0.0

    def write(self, out=sys.stdout):
        pass

    def write_now(self, mark=''):
        pass

    def add(self, timer):
        pass

    def __call__(self, name):
        return self

    def __enter__(self):
        pass

    def __exit__(self, *args):
        pass


nulltimer = NullTimer()


class DebugTimer(Timer):
    def __init__(self, print_levels=1000, comm=mpi.world, txt=sys.stdout):
        Timer.__init__(self, print_levels)
        ndigits = 1 + int(math.log10(comm.size))
        self.srank = '%0*d' % (ndigits, comm.rank)
        self.txt = txt

    def start(self, name):
        Timer.start(self, name)
        abstime = time.time()
        t = self.timers[tuple(self.running)] + abstime
        self.txt.write('T%s >> %15.8f %s (%7.5fs) started\n'
                       % (self.srank, abstime, name, t))

    def stop(self, name=None):
        if name is None:
            name = self.running[-1]
        abstime = time.time()
        t = self.timers[tuple(self.running)] + abstime
        self.txt.write('T%s << %15.8f %s (%7.5fs) stopped\n'
                       % (self.srank, abstime, name, t))
        Timer.stop(self, name)


class ParallelTimer(DebugTimer):
    """Like DebugTimer but writes timings from all ranks.

    Each rank writes to timings.<rank>.txt.  Also timings.metadata.txt
    will contain information about the parallelization layout.  The idea
    is that the output from this timer can be used for plots and to
    determine bottlenecks in the parallelization.

    See the tool gpaw-plot-parallel-timings."""
    def __init__(self, prefix='timings', flush=False):
        ndigits = len(str(mpi.world.size - 1))
        ranktxt = '%0*d' % (ndigits, mpi.world.rank)
        fname = '%s.%s.txt' % (prefix, ranktxt)
        txt = open(fname, 'w', buffering=1 if flush else -1)
        DebugTimer.__init__(self, comm=mpi.world, txt=txt)
        self.prefix = prefix

    def print_info(self, calc):
        """Print information about parallelization into a file."""
        fd = open('%s.metadata.txt' % self.prefix, 'w')
        DebugTimer.print_info(self, calc)
        wfs = calc.wfs

        # We won't have to type a lot if everyone just sends all their numbers.
        myranks = np.array([wfs.world.rank, wfs.kd.comm.rank,
                            wfs.bd.comm.rank, wfs.gd.comm.rank])
        allranks = None
        if wfs.world.rank == 0:
            allranks = np.empty(wfs.world.size * 4, dtype=int)
        wfs.world.gather(myranks, 0, allranks)
        if wfs.world.rank == 0:
            for itsranks in allranks.reshape(-1, 4):
                fd.write('r=%d k=%d b=%d d=%d\n' % tuple(itsranks))
        fd.close()


class StepTimer(Timer):
    """Step timer to print out timing used in computation steps.

    Use it like this::

      from gpaw.utilities.timing import StepTimer
      st = StepTimer()
      ...
      st.write_now('step 1')
      ...
      st.write_now('step 2')

    The parameter write_as_master_only can be used to force the timer to
    print from processess that are not the mpi master process.
    """

    def __init__(self, out=sys.stdout, name=None, write_as_master_only=True):
        Timer.__init__(self)
        if name is None:
            name = '<%s>' % sys._getframe(1).f_code.co_name
        self.name = name
        self.out = out
        self.alwaysprint = not write_as_master_only
        self.now = 'temporary now'
        self.start(self.now)

    def write_now(self, mark=''):
        self.stop(self.now)
        if self.alwaysprint or mpi.rank == 0:
            print(self.name, mark, self.get_time(self.now), file=self.out)
        self.out.flush()
        del self.timers[self.now]
        self.start(self.now)


class TAUTimer(Timer):
    """TAUTimer requires installation of the TAU Performance System
    http://www.cs.uoregon.edu/research/tau/home.php

    The TAU Python API will not output any data if there are any
    unmatched starts/stops in the code."""

    top_level = 'GPAW.calculator'  # TAU needs top level timer
    merge = True  # Requires TAU 2.19.2 or later

    def __init__(self):
        Timer.__init__(self)
        import pytau
        self.pytau = pytau
        self.tau_timers = {}
        pytau.setNode(mpi.rank)
        self.tau_timers[self.top_level] = pytau.profileTimer(self.top_level)
        pytau.start(self.tau_timers[self.top_level])

    def start(self, name):
        Timer.start(self, name)
        self.tau_timers[name] = self.pytau.profileTimer(name)
        self.pytau.start(self.tau_timers[name])

    def stop(self, name=None):
        Timer.stop(self, name)
        self.pytau.stop(self.tau_timers[name])

    def write(self, out=sys.stdout):
        Timer.write(self, out)
        if self.merge:
            self.pytau.dbMergeDump()
        else:
            self.pytau.stop(self.tau_timers[self.top_level])


class HPMTimer(Timer):
    """HPMTimer requires installation of the IBM BlueGene/P HPM
    middleware interface to the low-level UPC library. This will
    most likely only work at ANL's BlueGene/P. Must compile
    with GPAW_HPM macro in customize.py. Note that HPM_Init
    and HPM_Finalize are called in _gpaw.c and not in the Python
    interface. Timer must be called on all ranks in node, otherwise
    HPM will hang. Hence, we only call HPM_start/stop on a list
    subset of timers."""

    top_level = 'GPAW.calculator'  # HPM needs top level timer
    compatible = ['Initialization', 'SCF-cycle']

    def __init__(self):
        Timer.__init__(self)
        from _gpaw import hpm_start, hpm_stop
        self.hpm_start = hpm_start
        self.hpm_stop = hpm_stop
        hpm_start(self.top_level)

    def start(self, name):
        Timer.start(self, name)
        if name in self.compatible:
            self.hpm_start(name)

    def stop(self, name=None):
        Timer.stop(self, name)
        if name in self.compatible:
            self.hpm_stop(name)

    def write(self, out=sys.stdout):
        Timer.write(self, out)
        self.hpm_stop(self.top_level)


class CrayPAT_timer(Timer):
    """Interface to CrayPAT API. In addition to regular timers,
    the corresponding regions are profiled by CrayPAT. The gpaw-python has
    to be compiled under CrayPAT.
    """

    def __init__(self, print_levels=4):
        Timer.__init__(self, print_levels)
        from _gpaw import craypat_region_begin, craypat_region_end
        self.craypat_region_begin = craypat_region_begin
        self.craypat_region_end = craypat_region_end
        self.regions = {}
        self.region_id = 5  # leave room for regions in C

    def start(self, name):
        Timer.start(self, name)
        if name in self.regions:
            id = self.regions[name]
        else:
            id = self.region_id
            self.regions[name] = id
            self.region_id += 1
        self.craypat_region_begin(id, name)

    def stop(self, name=None):
        Timer.stop(self, name)
        id = self.regions[name]
        self.craypat_region_end(id)



from __future__ import print_function
import numpy as np

from gpaw.test import equal
from gpaw.grid_descriptor import GridDescriptor
from gpaw.spline import Spline
import gpaw.mpi as mpi
from gpaw.lfc import LocalizedFunctionsCollection as LFC
from gpaw.wavefunctions.pw import PWDescriptor, PWLFC
from gpaw.kpt_descriptor import KPointDescriptor


x = 2.0
rc = 3.5
r = np.linspace(0, rc, 100)

n = 40
a = 8.0
gd = GridDescriptor((n, n, n), (a, a, a), comm=mpi.serial_comm)

kpts = np.array([(0.25, 0.25, 0.0)])
kd = KPointDescriptor(kpts)
spos_ac = np.array([(0.15, 0.5, 0.95)])

pd = PWDescriptor(45, gd, complex, kd)

eikr = np.ascontiguousarray(np.exp(2j * np.pi * np.dot(np.indices(gd.N_c).T,
                                                         (kpts / gd.N_c).T).T)[0])

from gpaw.fftw import FFTPlan
print(FFTPlan)

for l in range(3):
    print(l)
    s = Spline(l, rc, 2 * x**1.5 / np.pi * np.exp(-x * r**2))

    lfc1 = LFC(gd, [[s]], kd, dtype=complex)
    lfc2 = PWLFC([[s]], pd)
    
    c_axi = {0: np.zeros((1, 2 * l + 1), complex)}
    c_axi[0][0, 0] = 1.9 - 4.5j
    c_axiv = {0: np.zeros((1, 2 * l + 1, 3), complex)}

    b1 = gd.zeros(1, dtype=complex)
    b2 = pd.zeros(1, dtype=complex)

    for lfc, b in [(lfc1, b1), (lfc2, b2)]:
        lfc.set_positions(spos_ac)
        lfc.add(b, c_axi, 0)

    b2 = pd.ifft(b2[0]) * eikr
    equal(abs(b2-b1[0]).max(), 0, 0.001)
    
    b1 = eikr[None]
    b2 = pd.fft(b1[0] * 0 + 1).reshape((1, -1))

    results = []
    results2 = []
    for lfc, b in [(lfc1, b1), (lfc2, b2)]:
        lfc.integrate(b, c_axi, 0)
        results.append(c_axi[0][0].copy())
        lfc.derivative(b, c_axiv, 0)
        results2.append(c_axiv[0][0].copy())
    equal(abs(np.ptp(results2, 0)).max(), 0, 1e-7)
    equal(abs(np.ptp(results, 0)).max(), 0, 3e-8)



"""Test of BLACS Redistributor.

Requires at least 8 MPI tasks.
"""


import numpy as np

from gpaw.band_descriptor import BandDescriptor
from gpaw.grid_descriptor import GridDescriptor
from gpaw.mpi import world, distribute_cpus
from gpaw.utilities import compiled_with_sl
from gpaw.utilities.scalapack import scalapack_set
from gpaw.blacs import parallelprint
from gpaw.kohnsham_layouts import BlacsBandLayouts

G = 120  # number of grid points (G x G x G)
N = 10  # number of bands

# B: number of band groups
# D: number of domains
B = 2
D = 2

n = N // B     # number of bands per group
assert n * B == N, 'n=%d, B=%d, N=%d' % (n, B, N)

h = 0.2        # grid spacing
a = h * G      # side length of box

# Set up communicators:
comms = distribute_cpus(parsize_domain=D,
                        parsize_bands=B,
                        nspins=1, nibzkpts=2)
domain_comm, kpt_comm, band_comm, block_comm = \
    [comms[name] for name in ['d', 'k', 'b', 'K']]
assert world.size == D*B*kpt_comm.size

if world.rank == 0:
    print('MPI: %d domains, %d band groups, %d kpts' % (domain_comm.size, band_comm.size, kpt_comm.size))

# Set up band and grid descriptors:
bd = BandDescriptor(N, band_comm, False)
gd = GridDescriptor((G, G, G), (a, a, a), True, domain_comm, parsize_c=D)

mcpus, ncpus, blocksize = 2, 2, 6

def blacs_diagonalize(ksl, H_Nn, U_nN, eps_n):
    # H_Nn must be lower triangular or symmetric,
    # but not upper triangular.
    # U_nN will be symmetric

    # U_Nn needs to be simultaneously compatible with:
    # 1. outdescriptor
    # 2. broadcast with gd.comm
    # We will do this with a dummy buffer U2_nN
    bmd = ksl.new_descriptor()
    H_nn = bmd.redistribute_output(H_Nn)
    ksl.diagonalize(H_nn, eps_n)
    U_nn = H_nn
    del H_nn
    U_nN[:] = bmd.redistribute_input(U_nn)

def blacs_inverse_cholesky(ksl, S_Nn, C_nN):
    # S_Nn must be upper triangular or symmetric,
    # but not lower triangular.
    # C_nN will be upper triangular.

    # S_Nn needs to be simultaneously compatible with:
    # 1. indescriptor
    # 2. outdescriptor
    # 3. broadcast with gd.comm
    # We will do this with a number of separate buffers.
    bmd = ksl.new_descriptor()
    S_nn = bmd.redistribute_output(S_Nn)
    ksl.inverse_cholesky(S_nn)
    C_nn = S_nn
    del S_nn
    C_nN[:] = bmd.redistribute_input(C_nn)

def main(seed=42, dtype=float):
    ksl = BlacsBandLayouts(gd, bd, block_comm, dtype, mcpus, ncpus, blocksize)
    nbands = bd.nbands
    mynbands = bd.mynbands

    # Diagonalize
    # We would *not* create H_Nn in the real-space code this way.
    # This is just for testing purposes.
    # Note after MPI_Reduce, only meaningful information on gd masters
    H_Nn = np.zeros((nbands, mynbands), dtype=dtype)
    U_nN = np.empty((mynbands, nbands), dtype=dtype)

    if ksl.Nndescriptor: # hack
        scalapack_set(ksl.Nndescriptor, H_Nn, 0.1, 75.0, 'L')
    else:
        assert gd.comm.rank != 0

    print("H_Nn")
    parallelprint(world, H_Nn)

    eps_n = np.zeros(bd.mynbands)
    blacs_diagonalize(ksl, H_Nn, U_nN, eps_n)
    print("U_nN")
    parallelprint(world, U_nN)
    print("eps_n")
    parallelprint(world, eps_n)

    # Inverse Cholesky
    S_Nn = np.zeros((nbands, mynbands), dtype=dtype)
    C_nN = np.empty((mynbands, nbands), dtype=dtype)

    if ksl.Nndescriptor: # hack
        scalapack_set(ksl.Nndescriptor, S_Nn, 0.1, 75.0, 'L')
    else:
        assert gd.comm.rank != 0

    print("S_Nn")
    parallelprint(world, S_Nn)
    blacs_inverse_cholesky(ksl, S_Nn, C_nN)
    print("C_nN")
    parallelprint(world, C_nN)

if __name__ in ['__main__', '__builtin__']:
    if not compiled_with_sl():
        print('Not built with ScaLAPACK. Test does not apply.')
    else:
        main(dtype=float)
        main(dtype=complex)





from __future__ import print_function
from ase import Atoms
from gpaw import GPAW, FermiDirac
from gpaw.xas import XAS, RecursionMethod
from gpaw.test import gen

# Generate setup for oxygen with half a core-hole:
gen('Si', name='hch1s', corehole=(1, 0, 0.5), gpernode=30)
a = 2.6
si = Atoms('Si', cell=(a, a, a), pbc=True)

import numpy as np
calc = GPAW(nbands=None,
            h=0.25,
            occupations=FermiDirac(width=0.05),
            setups={0: 'hch1s'})
def stopcalc():
    calc.scf.converged = 1
calc.attach(stopcalc, 1)
si.set_calculator(calc)
e = si.get_potential_energy()
niter = calc.get_number_of_iterations()
calc.write('si.gpw')

# restart from file
calc = GPAW('si.gpw')

import gpaw.mpi as mpi
if mpi.size == 1:
    xas = XAS(calc)
    x, y = xas.get_spectra()
else:
    x = np.linspace(0, 10, 50)

k = 2
calc.set(kpts=(k, k, k))
calc.initialize()
calc.set_positions(si)
assert calc.wfs.dtype == complex

r = RecursionMethod(calc)
r.run(40)
if mpi.size == 1:
    z = r.get_spectra(x)

if 0:
    import pylab as p
    p.plot(x, y[0])
    p.plot(x, sum(y))
    p.plot(x, z[0])
    p.show()

# 2p corehole
gen('Si', name='hch2p', corehole=(2, 1, 0.5), gpernode=30)
calc = GPAW(nbands=None,
            h=0.25,
            occupations=FermiDirac(width=0.05),
            setups={0: 'hch2p'})
si.set_calculator(calc)
def stopcalc():
    calc.scf.converged = True
calc.attach(stopcalc, 1)
e = si.get_potential_energy()
niter = calc.get_number_of_iterations()
calc.write('si_hch2p.gpw')




from ase import Atoms
from gpaw import GPAW, PW
a = 3.0
ec = 200
h2 = Atoms('H2', [[0, 0, 0], [0, 0, 0.8]],
           cell=[a, a, a], pbc=1)
h2.calc = GPAW(mode=PW(ec), txt='sc.txt')
e0 = h2.get_potential_energy()
h2.cell[1, 0] = a
e = h2.get_potential_energy()
assert abs(e - e0) < 0.001, e - e0



import numpy as np
from ase.build import molecule
from ase.data.vdw import vdw_radii
from gpaw import GPAW
from gpaw.cluster import Cluster
from gpaw.test import equal
from gpaw.solvation import (SolvationGPAW, EffectivePotentialCavity,
                            Power12Potential, LinearDielectric)

SKIP_REF_CALC = True

energy_eps = 0.0005 / 8.
forces_eps = 3e-2

h = 0.3
vac = 3.0
u0 = .180
T = 298.15
vdw_radii = vdw_radii.copy()
vdw_radii[1] = 1.09


def atomic_radii(atoms):
    return [vdw_radii[n] for n in atoms.numbers]

atoms = Cluster(molecule('H2O'))
atoms.minimal_box(vac, h)

convergence = {
    'energy': energy_eps,
    'forces': forces_eps ** 2,  # Force error is squared
    'density': 10.,
    'eigenstates': 10.,
}

if not SKIP_REF_CALC:
    atoms.calc = GPAW(xc='LDA', h=h, convergence=convergence)
    Eref = atoms.get_potential_energy()
    print(Eref)
    Fref = atoms.get_forces()
    print(Fref)
else:
    # setups: 0.9.11271, same settings as above
    Eref = -11.9932

    Fref = np.array(
        [[1.95122040e-12, -1.17770462e-12, -6.04993798e+00],
         [6.61270337e-14, 1.58227909e+00, 6.06605145e-02],
         [1.35947527e-13, -1.58227909e+00, 6.06605145e-02]])

atoms.calc = SolvationGPAW(
    xc='LDA', h=h, convergence=convergence,
    cavity=EffectivePotentialCavity(
        effective_potential=Power12Potential(atomic_radii=atomic_radii, u0=u0),
        temperature=T
    ),
    dielectric=LinearDielectric(epsinf=1.0),
)
Etest = atoms.get_potential_energy()
Eeltest = atoms.calc.get_electrostatic_energy()
Ftest = atoms.get_forces()
equal(Etest, Eref, energy_eps * atoms.calc.get_number_of_electrons())
equal(Ftest, Fref, forces_eps)
equal(Eeltest, Etest)



from __future__ import print_function
import itertools
import numpy as np
from gpaw.grid_descriptor import GridDescriptor
from gpaw.xc import XC
from gpaw.xc.libvdwxc import vdw_df_cx
from gpaw.mpi import world

def test(xc, tol=5e-10):
    N_c = np.array([10, 6, 4])
    # This test is totally serial.
    gd = GridDescriptor(N_c, N_c * 0.2, pbc_c=(1, 0, 1),
                        comm=world.new_communicator([world.rank]))
    actual_n = N_c - (1 - gd.pbc_c)

    gen = np.random.RandomState(17)
    n_sg = gd.zeros(2)
    n_sg[:] = gen.rand(*n_sg.shape)
    #sigma_xg = gd.zeros(3)
    #sigma_xg[:] = gen.random.rand(*sigma_xg.shape)

    if hasattr(xc, 'libvdwxc'):
        xc._nspins = 2
        xc.initialize_backend(gd)

    v_sg = gd.zeros(2)
    E = xc.calculate(gd, n_sg, v_sg)
    print('E', E)

    dn = 1e-6

    all_indices = itertools.product(range(2),
                                    range(1, actual_n[0], 2),
                                    range(0, actual_n[1], 2),
                                    range(0, actual_n[2], 2))

    for testindex in all_indices:
        n1_sg = n_sg.copy()
        n2_sg = n_sg.copy()
        v = v_sg[testindex]  * gd.dv
        n1_sg[testindex] -= dn
        n2_sg[testindex] += dn

        E1 = xc.calculate(gd, n1_sg, v_sg.copy())
        E2 = xc.calculate(gd, n2_sg, v_sg.copy())

        dedn = 0.5 * (E2 - E1) / dn
        err = abs(dedn - v)
        print('{}{} v={} fd={} err={}'.format(xc.name, list(testindex),
                                              v, dedn, err))
        assert err < tol, err

test(XC('PBE'))
test(vdw_df_cx())
test(vdw_df_cx(vdwcoef=0.0))
test(vdw_df_cx(vdwcoef=1e5), tol=2e-6)



# Test loading of sg15 setups as setups='sg15' and that the calculation
# agrees with PAW for the H2 eigenvalue.

from __future__ import print_function
from ase.build import molecule
from gpaw import GPAW, Davidson, Mixer
from gpaw.test.pseudopotential.H_sg15 import pp_text
from gpaw import setup_paths
setup_paths.insert(0, '.')
from gpaw.mpi import world

# We can't easily load a non-python file from the test suite.
# Therefore we load the pseudopotential from a Python file.
# But we want to test the pseudopotential search mechanism, therefore
# we immediately write it to a file:
if world.rank == 0:
    fd = open('H_ONCV_PBE-1.0.upf', 'w')
    fd.write(pp_text)
    fd.close()  # All right...
world.barrier()

system = molecule('H2')
system.center(vacuum=2.5)

def getkwargs():
    return dict(eigensolver=Davidson(4),
                mixer=Mixer(0.8, 5, 10.0),
                xc='oldPBE')

calc1 = GPAW(setups='sg15', h=0.13, **getkwargs())
system.set_calculator(calc1)
system.get_potential_energy()
eps1 = calc1.get_eigenvalues()

calc2 = GPAW(h=0.2, **getkwargs())
system.set_calculator(calc2)
system.get_potential_energy()
eps2 = calc2.get_eigenvalues()

err = eps2[0] - eps1[0]

# It is not the most accurate calculation ever, let's just make sure things
# are not completely messed up.
print('sg15 vs paw error', err)
assert abs(err) < 0.02  # 0.0055.... as of current test.



# Copyright (C) 2008 CAMd
# Please see the accompanying LICENSE file for further information.

from __future__ import division

import numpy as np

from gpaw.utilities.blas import gemm


def reshape(a_x, shape):
    """Get an ndarray of size shape from a_x buffer."""
    return a_x.ravel()[:np.prod(shape)].reshape(shape)


class MatrixOperator:
    """Base class for overlap and hamiltonian operators.

    Due to optimized BLAS usage, matrices are considered
    transposed both upon input and output.

    As both the overlap and Hamiltonian matrices are Hermitian, they
    can be considered as transposed *or* conjugated as compared to
    standard definitions.
    """

    nblocks = 1
    asynchronous = True
    hermitian = True

    def __init__(self, ksl, nblocks=None, asynchronous=None, hermitian=None):
        """The constructor now calculates the work array sizes, but does not
        allocate them. Here is a summary of the relevant variables and the
        cases handled.

        Given::

          J = nblocks              The number of blocks to divide bands and
                                   grid points into.
          N = mynbands             The number of bands on this MPI task
          M = np.ceil(N/float(J))  The number of bands in each block.
          G = gd.n_c.prod()        The number of grid points on this MPI task.
          g = np.ceil(G/float(J))  The number of grid points in each block.
          X and Q                  The workspaces to be calculated.

        Note that different values of J can lead to the same values of M
        and G. Q is relatively simple to calculate, symmetric case needs
        *roughly* half as much storage space as the non-symmetric case.
        X is much more difficult. Read below.

        X is the band index of the workspace array. It is allocated in units
        of the wavefunctions. Here is the condition on X and some intermediate
        variables::

              M >  0        At least one band in a block
              X >= M        Blocking over band index must have enough space.
          X * G >= N * g    Blocking over grid index must have enough space.

        There are two different parallel matrix multiples here:
        1. calculate_matrix_elements contracts on grid index
        2. matrix_multiply contracts on the band index

        We simply needed to make sure that we have enough workspace for
        both of these multiples since we re-use the workspace arrays.

        Cases::

          Simplest case is G % J = M % J = 0: X = M.
          
          If g * N > M * G, then we need to increase the buffer size by one
          wavefunction unit greater than the simple case, thus X = M + 1.

        """
        self.bd = ksl.bd
        self.gd = ksl.gd
        self.block_comm = ksl.block_comm
        self.bmd = ksl.new_descriptor()  # XXX take hermitian as argument?
        self.dtype = ksl.dtype
        self.buffer_size = ksl.buffer_size
        if nblocks is not None:
            self.nblocks = nblocks
        if asynchronous is not None:
            self.asynchronous = asynchronous
        if hermitian is not None:
            self.hermitian = hermitian

        # default for work spaces
        self.A_nn = None
        self.work1_xG = None
        self.work2_xG = None

        mynbands = self.bd.mynbands
        ngroups = self.bd.comm.size
        G = self.gd.n_c.prod()

        # If buffer_size keyword exist, use it to calculate closest
        # corresponding value of nblocks. An *attempt* is made
        # such that actual buffer size used does not exceed the
        # value specified by buffer_size.
        # Maximum allowable buffer_size corresponds to nblock = 1
        # which is all the wavefunctions.
        # Give error if the buffer_size is so small that it cannot
        # contain a single wavefunction
        if self.buffer_size is not None:  # buffersize is in KiB
            sizeof_single_wfs = float(self.gd.bytecount(self.dtype))
            numberof_wfs = self.buffer_size * 1024 / sizeof_single_wfs
            assert numberof_wfs > 0  # buffer_size is too small
            self.nblocks = max(int(mynbands // numberof_wfs), 1)
            
        # Calculate Q and X for allocating arrays later
        self.X = 1  # not used for ngroups == 1 and J == 1
        self.Q = 1
        J = self.nblocks
        M = int(np.ceil(mynbands / float(J)))
        g = int(np.ceil(G / float(J)))
        assert M > 0  # must have at least one wave function in a block

        if ngroups == 1 and J == 1:
            pass
        else:
            if g * mynbands > M * G:  # then more space is needed
                self.X = M + 1
                assert self.X * G >= g * mynbands
            else:
                self.X = M

    def allocate_arrays(self):
        ngroups = self.bd.comm.size
        mynbands = self.bd.mynbands
        dtype = self.dtype
        self.A_nn = self.bmd.zeros(dtype=dtype)

        if ngroups == 1 and self.nblocks == 1:
            self.work1_xG = self.gd.empty(mynbands, self.dtype)
        else:
            self.work1_xG = self.gd.empty(self.X, self.dtype)
            self.work2_xG = self.gd.empty(self.X, self.dtype)

    def estimate_memory(self, mem, dtype):
        ngroups = self.bd.comm.size
        count = ngroups * self.bd.mynbands**2

        # Code semipasted from allocate_work_arrays
        if ngroups > 1:
            mem.subnode('A_qnn', count * mem.itemsize[dtype])

        if ngroups == 1 and self.nblocks == 1:
            mem.subnode('work1_xG',
                        self.bd.mynbands * self.gd.bytecount(self.dtype))
        else:
            mem.subnode('work1_xG', self.X * self.gd.bytecount(self.dtype))
            mem.subnode('work2_xG', self.X * self.gd.bytecount(self.dtype))

        self.bmd.estimate_memory(mem.subnode('Band Matrices'), dtype)

    def _initialize_cycle(self, sbuf_mG, rbuf_mG, sbuf_nI, rbuf_nI, auxiliary):
        """Initializes send/receive cycle of pseudo wave functions, as well as
        an optional auxiliary send/receive cycle of corresponding projections.
        Low-level helper function. Results in the following communications::

                        Rank below            This rank            Rank above
          Asynchronous: ... o/i  <-- sbuf_mG --  o/i  <-- rbuf_mG --  o/i ...
          Synchronous:     blank                blank                blank

          Auxiliary:    ... o/i  <-- sbuf_nI --  o/i  <-- rbuf_nI --  o/i ...

        A letter 'o' signifies a non-blocking send and 'i' a matching receive.


        Parameters:

        sbuf_mG: ndarray
            Send buffer for the outgoing set of pseudo wave functions.
        rbuf_mG: ndarray
            Receive buffer for the incoming set of pseudo wave functions.
        sbuf_nI: ndarray, ignored if not auxiliary
            Send buffer for the outgoing set of atomic projector overlaps.
        rbuf_nI: ndarray, ignored if not auxiliary
            Receive buffer for the incoming set of atomic projector overlaps.
        auxiliary: bool
            Determines whether to initiate the auxiliary send/receive cycle.

        """
        band_comm = self.bd.comm
        rankm = (band_comm.rank - 1) % band_comm.size
        rankp = (band_comm.rank + 1) % band_comm.size
        self.req, self.req2 = [], []

        # If asynchronous, non-blocking send/receives of psit_nG's start here.
        if self.asynchronous:
            self.req.append(band_comm.send(sbuf_mG, rankm, 11, False))
            self.req.append(band_comm.receive(rbuf_mG, rankp, 11, False))

        # Auxiliary asynchronous cycle, also send/receive of P_ani's.
        if auxiliary:
            self.req2.append(band_comm.send(sbuf_nI, rankm, 31, False))
            self.req2.append(band_comm.receive(rbuf_nI, rankp, 31, False))

    def _finish_cycle(self, sbuf_mG, rbuf_mG, sbuf_nI, rbuf_nI, auxiliary):
        """Completes a send/receive cycle of pseudo wave functions, as well as
        an optional auxiliary send/receive cycle of corresponding projections.
        Low-level helper function. Results in the following communications::

                        Rank below            This rank            Rank above
          Asynchronous: ... w/w  <-- sbuf_mG --  w/w  <-- rbuf_mG --  w/w ...
          Synchronous:  ... O/I  <-- sbuf_mG --  O/I  <-- rbuf_mG --  O/I ...

          Auxiliary:    ... w/w  <-- sbuf_nI --  w/w  <-- rbuf_nI --  w/w ...

        A letter 'w' signifies wait for initialized non-blocking communication.
        The letter 'O' signifies a blocking send and 'I' a matching receive.


        Parameters:

        Same as _initialize_cycle.

        Returns:

        sbuf_mG: ndarray
            New send buffer with the received set of pseudo wave functions.
        rbuf_mG: ndarray
            New receive buffer (has the sent set of pseudo wave functions).
        sbuf_nI: ndarray, same as input if not auxiliary
            New send buffer with the received set of atomic projector overlaps.
        rbuf_nI: ndarray, same as input if not auxiliary
            New receive buffer (has the sent set of atomic projector overlaps).

        """
        band_comm = self.bd.comm
        rankm = (band_comm.rank - 1) % band_comm.size
        rankp = (band_comm.rank + 1) % band_comm.size

        # If syncronous, blocking send/receives of psit_nG's carried out here.
        if self.asynchronous:
            assert len(self.req) == 2, 'Expected asynchronous request pairs.'
            band_comm.waitall(self.req)
        else:
            assert len(self.req) == 0, 'Got unexpected asynchronous requests.'
            band_comm.sendreceive(sbuf_mG, rankm, rbuf_mG, rankp, 11, 11)
        sbuf_mG, rbuf_mG = rbuf_mG, sbuf_mG

        # Auxiliary asynchronous cycle, also wait for P_ani's.
        if auxiliary:
            assert len(self.req2) == 2, 'Expected asynchronous request pairs.'
            band_comm.waitall(self.req2)
            sbuf_nI, rbuf_nI = rbuf_nI, sbuf_nI

        return sbuf_mG, rbuf_mG, sbuf_nI, rbuf_nI

    def calculate_matrix_elements(self, psit1_nG, P1_ani, A, dA,
                                  psit2_nG=None, P2_ani=None):
        """Calculate matrix elements for A-operator.

        Results will be put in the *A_nn* array::

                                    ___
                    ~ 2  ^  ~ 1     \     ~   ~a    a   ~a  ~
           A    = <psi  |A|psi  > +  )  <psi |p > dA   <p |psi >
            nn'       n      n'     /___    n  i    ii'  i'   n'
                                     aii'

        Fills in the lower part of *A_nn*, but only on domain and band masters.


        Parameters:

        psit_nG: ndarray
            Set of vectors in which the matrix elements are evaluated.
        P_ani: dict
            Dictionary of projector overlap integrals P_ni = <p_i | psit_nG>.
        A: function
            Functional form of the operator A which works on psit_nG.
            Must accept and return an ndarray of the same shape as psit_nG.
        dA: function
            Operator which works on | phi_i >.  Must accept atomic
            index a and P_ni and return an ndarray with the same shape
            as P_ni, thus representing P_ni multiplied by dA_ii.

        """
        band_comm = self.bd.comm
        domain_comm = self.gd.comm
        block_comm = self.block_comm

        B = band_comm.size
        J = self.nblocks
        N = self.bd.mynbands
        M = int(np.ceil(N / float(J)))

        if psit2_nG is None:
            psit2_nG = psit1_nG
            hermitian = self.hermitian
        else:
            hermitian = False
        
        if P2_ani is None:
            P2_ani = P1_ani

        if self.A_nn is None:
            self.allocate_arrays()

        A_NN = self.A_nn

        if B == 1 and J == 1:
            # Simple case:
            Apsit_nG = A(psit2_nG)
            self.gd.integrate(psit1_nG, Apsit_nG, hermitian=hermitian,
                              _transposed_result=A_NN)
            for a, P1_ni in P1_ani.items():
                P2_ni = P2_ani[a]
                gemm(1.0, P1_ni, dA(a, P2_ni), 1.0, A_NN, 'c')
            domain_comm.sum(A_NN, 0)
            return self.bmd.redistribute_output(A_NN)
        
        # Now it gets nasty! We parallelize over B groups of bands and
        # each band group is blocked in J smaller slices (less memory).

        if hermitian:
            Q = B // 2 + 1
        else:
            Q = B
        
        # Buffer for storage of blocks of calculated matrix elements.
        if B == 1:
            A_qnn = A_NN.reshape((1, N, N))
        else:
            A_qnn = np.zeros((Q, N, N), self.dtype)

        # Buffers for send/receive of operated-on versions of P_ani's.
        sbuf_nI = rbuf_nI = None
        if P2_ani:
            sbuf_nI = np.hstack([dA(a, P_ni) for a, P_ni in P2_ani.items()])
            sbuf_nI = np.ascontiguousarray(sbuf_nI)
            if B > 1:
                rbuf_nI = np.empty_like(sbuf_nI)

        work1_xG = reshape(self.work1_xG, (self.X,) + psit1_nG.shape[1:])
        work2_xG = reshape(self.work2_xG, (self.X,) + psit1_nG.shape[1:])

        # Because of the amount of communication involved, we need to
        # be syncronized up to this point but only on the 1D band_comm
        # communication ring
        band_comm.barrier()
        while M * J >= N + M:  # remove extra slice(s)
            J -= 1
        assert 0 < J * M < N + M

        for j in range(J):
            n1 = j * M
            n2 = n1 + M
            if n2 > N:
                n2 = N
                M = n2 - n1
            psit_mG = psit2_nG[n1:n2]
            temp_mG = A(psit_mG)
            sbuf_mG = temp_mG[:M]  # necessary only for last slice
            rbuf_mG = work2_xG[:M]
            cycle_P_ani = (j == J - 1 and P1_ani)

            for q in range(Q):
                A_nn = A_qnn[q]
                A_mn = A_nn[n1:n2]

                # Start sending currently buffered kets to rank below
                # and receiving next set of kets from rank above us.
                # If we're at the last slice, start cycling P_ani too.
                if q < Q - 1:
                    self._initialize_cycle(sbuf_mG, rbuf_mG,
                                           sbuf_nI, rbuf_nI, cycle_P_ani)

                # Calculate pseudo-braket contributions for the current slice
                # of bands in the current mynbands x mynbands matrix block.
                # The special case may no longer be valid when. Better to be
                # conservative, than to risk it. Moreover, this special case
                # seems is an accident waiting to happen. Always doing the
                # more general case is safer.
                # if q == 0 and self.hermitian and not self.bd.strided:
                #    # Special case, we only need the lower part:
                #     self._pseudo_braket(psit_nG[:n2], sbuf_mG, A_mn[:, :n2])
                # else:
                self.gd.integrate(psit1_nG, sbuf_mG, hermitian=False,
                                  _transposed_result=A_mn)

                # If we're at the last slice, add contributions from P_ani's.
                if cycle_P_ani:
                    I1 = 0
                    for P1_ni in P1_ani.values():
                        I2 = I1 + P1_ni.shape[1]
                        gemm(1.0, P1_ni, sbuf_nI[:, I1:I2],
                             1.0, A_nn, 'c')
                        I1 = I2

                # Wait for all send/receives to finish before next iteration.
                # Swap send and receive buffer such that next becomes current.
                # If we're at the last slice, also finishes the P_ani cycle.
                if q < Q - 1:
                    sbuf_mG, rbuf_mG, sbuf_nI, rbuf_nI = self._finish_cycle(
                        sbuf_mG, rbuf_mG, sbuf_nI, rbuf_nI, cycle_P_ani)

                # First iteration was special because we had the ket to ourself
                if q == 0:
                    rbuf_mG = work1_xG[:M]

        domain_comm.sum(A_qnn, 0)

        if B == 1:
            return self.bmd.redistribute_output(A_NN)

        if domain_comm.rank == 0:
            self.bmd.assemble_blocks(A_qnn, A_NN, hermitian)

        # Because of the amount of communication involved, we need to
        # be syncronized up to this point.
        block_comm.barrier()
        return self.bmd.redistribute_output(A_NN)
        
    def matrix_multiply(self, C_NN, psit_nG, P_ani=None, out_nG=None):
        """Calculate new linear combinations of wave functions.

        Results will be put in the *P_ani* dict and a new psit_nG returned::

                     __                                __
            ~       \       ~           ~a  ~         \       ~a  ~
           psi  <--  ) C   psi    and  <p |psi >  <--  ) C   <p |psi >
              n     /__ nn'   n'         i    n       /__ nn'  i    n'
                     n'                                n'


        Parameters:

        C_NN: ndarray
            Matrix representation of the requested linear combinations. Even
            with a hermitian operator, this matrix need not be self-adjoint.
            However, unlike the results from calculate_matrix_elements, it is
            assumed that all matrix elements are filled in (use e.g. tri2full).
        psit_nG: ndarray
            Set of vectors in which the matrix elements are evaluated.
        P_ani: dict
            Dictionary of projector overlap integrals P_ni = <p_i | psit_nG>.

        """

        if self.A_nn is None:
            self.allocate_arrays()

        band_comm = self.bd.comm
        B = band_comm.size
        J = self.nblocks
        N = self.bd.mynbands

        C_NN = self.bmd.redistribute_input(C_NN)

        if B == 1 and J == 1:
            # Simple case:
            work_nG = reshape(self.work1_xG, psit_nG.shape)
            if out_nG is None:
                out_nG = work_nG
                out_nG[:] = 117  # gemm may not like nan's
            elif out_nG is psit_nG:
                work_nG[:] = psit_nG
                psit_nG = work_nG
            self.gd.gemm(1.0, psit_nG, C_NN, 0.0, out_nG)
            if P_ani:
                for P_ni in P_ani.values():
                    gemm(1.0, P_ni.copy(), C_NN, 0.0, P_ni)
            return out_nG
        
        # Now it gets nasty! We parallelize over B groups of bands and
        # each grid chunk is divided in J smaller slices (less memory).

        Q = B  # always non-hermitian XXX
        rank = band_comm.rank
        shape = psit_nG.shape
        psit_nG = psit_nG.reshape(N, -1)
        G = psit_nG.shape[1]  # number of grid-points
        g = int(np.ceil(G / float(J)))

        # Buffers for send/receive of pre-multiplication versions of P_ani's.
        sbuf_nI = rbuf_nI = None
        if P_ani:
            sbuf_nI = np.hstack([P_ni for P_ni in P_ani.values()])
            sbuf_nI = np.ascontiguousarray(sbuf_nI)
            if B > 1:
                rbuf_nI = np.empty_like(sbuf_nI)

        # Because of the amount of communication involved, we need to
        # be syncronized up to this point but only on the 1D band_comm
        # communication ring
        band_comm.barrier()
        while g * J >= G + g:  # remove extra slice(s)
            J -= 1
        assert 0 < g * J < G + g

        work1_xG = reshape(self.work1_xG, (self.X,) + psit_nG.shape[1:])
        work2_xG = reshape(self.work2_xG, (self.X,) + psit_nG.shape[1:])

        for j in range(J):
            G1 = j * g
            G2 = G1 + g
            if G2 > G:
                G2 = G
                g = G2 - G1
            sbuf_ng = reshape(work1_xG, (N, g))
            rbuf_ng = reshape(work2_xG, (N, g))
            sbuf_ng[:] = psit_nG[:, G1:G2]
            beta = 0.0
            cycle_P_ani = (j == J - 1 and P_ani)
            for q in range(Q):
                # Start sending currently buffered kets to rank below
                # and receiving next set of kets from rank above us.
                # If we're at the last slice, start cycling P_ani too.
                if q < Q - 1:
                    self._initialize_cycle(sbuf_ng, rbuf_ng,
                                           sbuf_nI, rbuf_nI, cycle_P_ani)

                # Calculate wave-function contributions from the current slice
                # of grid data by the current mynbands x mynbands matrix block.
                C_nn = self.bmd.extract_block(C_NN, (rank + q) % B, rank)
                self.gd.gemm(1.0, sbuf_ng, C_nn, beta, psit_nG[:, G1:G2])

                # If we're at the last slice, add contributions to P_ani's.
                if cycle_P_ani:
                    I1 = 0
                    for P_ni in P_ani.values():
                        I2 = I1 + P_ni.shape[1]
                        gemm(1.0, sbuf_nI[:, I1:I2], C_nn, beta, P_ni)
                        I1 = I2

                # Wait for all send/receives to finish before next iteration.
                # Swap send and receive buffer such that next becomes current.
                # If we're at the last slice, also finishes the P_ani cycle.
                if q < Q - 1:
                    sbuf_ng, rbuf_ng, sbuf_nI, rbuf_nI = self._finish_cycle(
                        sbuf_ng, rbuf_ng, sbuf_nI, rbuf_nI, cycle_P_ani)

                # First iteration was special because we initialized the kets
                if q == 0:
                    beta = 1.0

        psit_nG.shape = shape
        return psit_nG



from ase import Atoms
from gpaw import GPAW, PoissonSolver
from gpaw.mixer import Mixer
from gpaw.test import equal
from gpaw.test import gen

a = 6.0
c = a / 2
# d = 1.8

elements = ['C', 'Be']
results = [0.243, 9.9773]
electrons = [6, 3]
charges = [0, 1]


for symbol in elements:
    xcname = '1.0_LDA_K_TF+1.0_LDA_X'
    g = gen(symbol, xcname=xcname, scalarrel=False, orbital_free=True,
            gpernode=75)

for element, result, e, charge in zip(elements, results, electrons, charges):
    atom = Atoms(element,
                 positions=[(c, c, c)],
                 cell=(a, a, a))

    mixer = Mixer(0.3, 5, 1)
    calc = GPAW(gpts=(32, 32, 32),
                txt='-', xc=xcname,
                poissonsolver=PoissonSolver(eps=1e-6),
                eigensolver='cg', mixer=mixer, charge=charge)

    atom.set_calculator(calc)

    E = atom.get_total_energy()
    n = calc.get_all_electron_density()

    dv = atom.get_volume() / calc.get_number_of_grid_points().prod()
    I = n.sum() * dv / 2**3

    equal(I, e, 1.0e-6)
    equal(result, E, 1.0e-2)



from ase import Atoms
from gpaw import GPAW
from gpaw.mpi import size
d = 4.0 / 2**0.5
ndomains = size // 8 + 1
calc = GPAW(h=d / 16, kpts=(17, 1, 1), parallel={'domain': ndomains,
                                                 'band': 1})
chain = Atoms('Al', cell=(d, 5, 5), pbc=True, calculator=calc)
e = chain.get_potential_energy()
print(e)
assert abs(e - -1.8182) < 0.0005
assert calc.wfs.kd.comm.size * ndomains == size
calc.write('al_chain', 'all')



"""This module is used to generate atomic orbital basis sets."""
from __future__ import print_function
import sys
try:
    from StringIO import StringIO  # Python 2
except ImportError:
    from io import StringIO

import numpy as np
from ase.units import Hartree
from ase.utils import devnull

from gpaw import __version__ as version
from gpaw.utilities import divrl
from gpaw.atom.generator import Generator
from gpaw.atom.all_electron import AllElectron
from gpaw.atom.configurations import parameters
from gpaw.basis_data import Basis, BasisFunction, get_basis_name
from gpaw.atom.radialgd import (AERadialGridDescriptor,
                                EquidistantRadialGridDescriptor)


def get_basis_l(f_j, l_j, Nv):
    N = 0
    lvalues = []
    j_l = []
    Nl_j = len(l_j)
    while N < Nv:
        Nl_j -= 1
        if f_j[Nl_j] > 0:
            lvalues.append(l_j[Nl_j])
            N += f_j[Nl_j]
            j_l.append(Nl_j)
    assert N == Nv

    if min(lvalues) != 0:  # always include s-orbital !
        lvalues = lvalues + [0]
        reversed_l_j = list(l_j)
        reversed_l_j.reverse()
        j = len(reversed_l_j) - reversed_l_j.index(0) - 1
        j_l.append(j)

    return [j_l, lvalues]


def get_gaussianlike_basis_function(rgd, l, rchar, gcut):
    rcut = rgd.r_g[gcut]
    gaussian = QuasiGaussian(1.0 / rchar**2, rcut)
    r_g = rgd.r_g[:gcut + 1]
    phit_g = gaussian(r_g) * r_g**l
    phit_g[-1] = 0.0
    norm = np.dot(rgd.dr_g[:gcut + 1], (r_g * phit_g)**2)**0.5
    phit_g /= norm
    norm2 = np.dot(rgd.dr_g[:gcut + 1], (r_g * phit_g)**2)**0.5
    assert abs(1.0 - norm2) < 1e-10
    return phit_g


def make_split_valence_basis_function(r_g, psi_g, l, gcut):
    """Get polynomial which joins psi smoothly at rcut.

    Returns an array of function values f(r) * r, where::

              l           2
      f(r) = r  * (a - b r ),  r < rcut
      f(r) = psi(r),           r >= rcut

    where a and b are determined such that f(r) is continuous and
    differentiable at rcut.  The parameter psi should be an atomic
    orbital.
    """
    r1 = r_g[gcut]  # ensure that rcut is moved to a grid point
    r2 = r_g[gcut + 1]
    y1 = psi_g[gcut] / r_g[gcut]
    y2 = psi_g[gcut + 1] / r_g[gcut + 1]
    b = - (y2 / r2**l - y1 / r1**l) / (r2**2 - r1**2)
    a = (y1 / r1**l + b * r1**2)
    psi_g2 = r_g**(l + 1) * (a - b * r_g**2)
    psi_g2[gcut:] = psi_g[gcut:]
    return psi_g2


def rsplit_by_norm(rgd, l, u, tailnorm_squared, txt):
    """Find radius outside which remaining tail has a particular norm."""
    norm_squared = np.dot(rgd.dr_g, u * u)
    partial_norm_squared = 0.
    i = len(u) - 1
    absolute_tailnorm_squared = tailnorm_squared * norm_squared
    while partial_norm_squared < absolute_tailnorm_squared:
        # Integrate backwards.  This is important since the pseudo
        # wave functions have strange behaviour near the core.
        partial_norm_squared += rgd.dr_g[i] * u[i]**2
        i -= 1
    rsplit = rgd.r_g[i + 1]
    msg = ('Tail norm %.03f :: rsplit=%.02f Bohr' %
           ((partial_norm_squared / norm_squared)**0.5, rsplit))
    print(msg, file=txt)
    gsplit = rgd.floor(rsplit)
    splitwave = make_split_valence_basis_function(rgd.r_g, u, l, gsplit)
    return rsplit, partial_norm_squared, splitwave


class BasisMaker:
    """Class for creating atomic basis functions."""
    def __init__(self, generator, name=None, run=True, gtxt='-',
                 non_relativistic_guess=False, xc='PBE',
                 save_setup=False):

        if isinstance(generator, str):  # treat 'generator' as symbol
            generator = Generator(generator, scalarrel=True,
                                  xcname=xc, txt=gtxt,
                                  nofiles=True)
            generator.N *= 4
        self.generator = generator
        self.rgd = AERadialGridDescriptor(generator.beta / generator.N,
                                          1.0 / generator.N, generator.N,
                                          default_spline_points=100)
        self.name = name

        if run:
            if non_relativistic_guess:
                ae0 = AllElectron(generator.symbol, scalarrel=False,
                                  nofiles=False, txt=gtxt, xcname=xc)
                ae0.N = generator.N
                ae0.beta = generator.beta
                ae0.run()
                # Now files will be stored such that they can
                # automagically be used by the next run()
            setup = generator.run(write_xml=False, use_restart_file=False,
                                  name=name,
                                  **parameters[generator.symbol])

            if save_setup:
                setup.write_xml()
        else:
            if save_setup:
                raise ValueError('cannot save setup here because setup '
                                 'was already generated before basis '
                                 'generation.')

    def smoothify(self, psi_mg, l):
        r"""Generate pseudo wave functions from all-electron ones.

        The pseudo wave function is::

                                   ___
               ~                   \    /   ~             \    ~     ~
            | psi  > = | psi  > +   )  | | phi > - | phi > ) < p  | psi > ,
                 m          m      /__  \     i         i /     i      m
                                    i

        where the scalar products are found by solving::

                            ___
              ~             \     ~             ~     ~
            < p | psi  > =   )  < p  | phi  > < p  | psi  > .
               i     m      /__    i      j      j      m
                             j

        In order to ensure smoothness close to the core, the
        all-electron wave function and partial wave are then
        multiplied by a radial function which approaches 0 near the
        core, such that the pseudo wave function approaches::

                        ___
               ~        \      ~       ~     ~
            | psi  > =   )  | phi >  < p  | psi >    (for r << rcut),
                 m      /__      i      i      m
                         i

        which is exact if the projectors/pseudo partial waves are complete.
        """
        if psi_mg.ndim == 1:
            return self.smoothify(psi_mg[None], l)[0]

        g = self.generator
        u_ng = g.u_ln[l]
        q_ng = g.q_ln[l]
        s_ng = g.s_ln[l]

        Pi_nn = np.dot(g.r * q_ng, u_ng.T)
        Q_nm = np.dot(g.r * q_ng, psi_mg.T)
        Qt_nm = np.linalg.solve(Pi_nn, Q_nm)

        # Weight-function for truncating all-electron parts smoothly near core
        gmerge = g.r2g(g.rcut_l[l])
        w_g = np.ones(g.r.shape)
        w_g[0:gmerge] = (g.r[0:gmerge] / g.r[gmerge])**2.
        w_g = w_g[None]

        psit_mg = psi_mg * w_g + np.dot(Qt_nm.T, s_ng - u_ng * w_g)
        return psit_mg

    def make_orbital_vector(self, j, rcut, vconf=None):
        """Returns a smooth basis vector given an all-electron one."""
        l = self.generator.l_j[j]
        psi_g, e = self.generator.solve_confined(j, rcut, vconf)
        psit_g = self.smoothify(psi_g, l)
        return psit_g

    def rcut_by_energy(self, j, esplit=.1, tolerance=.1, rguess=6.,
                       vconf_args=None):
        """Find confinement cutoff corresponding to given orbital energy shift.

        Creates a confinement potential for the orbital given by j,
        such that the confined-orbital energy is (emin to emax) eV larger
        than the free-orbital energy."""
        g = self.generator
        e_base = g.e_j[j]
        rc = rguess

        if vconf_args is None:
            vconf = None
        else:
            amplitude, ri_rel = vconf_args
            vconf = g.get_confinement_potential(amplitude, ri_rel * rc, rc)

        psi_g, e = g.solve_confined(j, rc, vconf)
        de_min, de_max = esplit / Hartree, (esplit + tolerance) / Hartree

        rmin = 0.
        rmax = g.r[-1]

        de = e - e_base
        # print '--------'
        # print 'Start bisection'
        # print'e_base =',e_base
        # print 'e =',e
        # print '--------'
        while de < de_min or de > de_max:
            if de < de_min:  # Move rc left -> smaller cutoff, higher energy
                rmax = rc
                rc = (rc + rmin) / 2.
            else:  # Move rc right
                rmin = rc
                rc = (rc + rmax) / 2.
            if vconf is not None:
                vconf = g.get_confinement_potential(amplitude, ri_rel * rc, rc)
            psi_g, e = g.solve_confined(j, rc, vconf)
            de = e - e_base
            # print 'rc = %.03f :: e = %.03f :: de = %.03f' % (rc, e*Hartree,
            #                                                  de*Hartree)
            # if rmin - rmax < 1e-
            if g.r2g(rmax) - g.r2g(rmin) <= 1:  # adjacent points
                break  # cannot meet tolerance due to grid resolution
        # print 'Done!'
        return psi_g, e, de, vconf, rc

    def generate(self, zetacount=2, polarizationcount=1,
                 tailnorm=(0.16, 0.3, 0.6), energysplit=0.1, tolerance=1.0e-3,
                 referencefile=None, referenceindex=None, rcutpol_rel=1.0,
                 rcutmax=20.0,
                 rcharpol_rel=None,
                 vconf_args=(12.0, 0.6), txt='-',
                 include_energy_derivatives=False,
                 # lvalues=None, # XXX clean up some of these!
                 jvalues=None,
                 l_pol=None):
        """Generate an entire basis set.

        This is a high-level method which will return a basis set
        consisting of several different basis vector types.

        Parameters:

        ===================== =================================================
        ``zetacount``         Number of basis functions per occupied orbital
        ``polarizationcount`` Number of polarization functions
        ``tailnorm``          List of tail norms for split-valence scheme
        ``energysplit``       Energy increase defining confinement radius (eV)
        ``tolerance``         Tolerance of energy split (eV)
        ``referencefile``     gpw-file used to generate polarization function
        ``referenceindex``    Index in reference system of relevant atom
        ``rcutpol_rel``       Polarization rcut relative to largest other rcut
        ``rcutmax``           No cutoff will be greater than this value
        ``vconf_args``        Parameters (alpha, ri/rc) for conf. potential
        ``txt``               Log filename or '-' for stdout
        ===================== =================================================

        Returns a fully initialized Basis object.
        """

        if txt == '-':
            txt = sys.stdout
        elif txt is None:
            txt = devnull

        if isinstance(tailnorm, float):
            tailnorm = (tailnorm,)
        if 1 + len(tailnorm) < max(polarizationcount, zetacount):
            raise ValueError(
                'Needs %d tail norm values, but only %d are specified' %
                (max(polarizationcount, zetacount) - 1, len(tailnorm)))

        textbuffer = StringIO()

        class TeeStream:  # quick hack to both write and save output
            def __init__(self, out1, out2):
                self.out1 = out1
                self.out2 = out2

            def write(self, string):
                self.out1.write(string)
                self.out2.write(string)

        txt = TeeStream(txt, textbuffer)

        if vconf_args is not None:
            amplitude, ri_rel = vconf_args

        g = self.generator
        rgd = self.rgd

        njcore = g.njcore
        n_j = g.n_j[njcore:]
        l_j = g.l_j[njcore:]
        f_j = g.f_j[njcore:]

        if jvalues is None:
            jvalues = []
            sortkeys = []
            for j in range(len(n_j)):
                if f_j[j] == 0 and l_j[j] != 0:
                    continue
                jvalues.append(j)
                sortkeys.append(l_j[j])

            # Now order jvalues by l
            #
            # Use a stable sort so the energy ordering within each
            # angular momentum is guaranteed to be preserved
            args = np.argsort(sortkeys, kind='mergesort')
            jvalues = np.array(jvalues)[args]

        fulljvalues = [njcore + j for j in jvalues]

        if isinstance(energysplit, float):
            energysplit = [energysplit] * len(jvalues)

        title = '%s Basis functions for %s' % (g.xcname, g.symbol)
        print(title, file=txt)
        print('=' * len(title), file=txt)

        singlezetas = []
        energy_derivative_functions = []
        multizetas = [[] for i in range(zetacount - 1)]
        polarization_functions = []

        splitvalencedescr = 'split-valence wave, fixed tail norm'
        derivativedescr = 'derivative of sz wrt. (ri/rc) of potential'

        for vj, fullj, esplit in zip(jvalues, fulljvalues, energysplit):
            l = l_j[vj]
            n = n_j[vj]
            assert n > 0
            orbitaltype = str(n) + 'spdf'[l]
            msg = 'Basis functions for l=%d, n=%d' % (l, n)
            print(file=txt)
            print(msg + '\n', '-' * len(msg), file=txt)
            print(file=txt)
            if vconf_args is None:
                adverb = 'sharply'
            else:
                adverb = 'softly'
            print('Zeta 1: %s confined pseudo wave,' % adverb, end=' ',
                  file=txt)

            u, e, de, vconf, rc = self.rcut_by_energy(fullj, esplit,
                                                      tolerance,
                                                      vconf_args=vconf_args)
            if rc > rcutmax:
                rc = rcutmax  # scale things down
                if vconf is not None:
                    vconf = g.get_confinement_potential(amplitude, ri_rel * rc,
                                                        rc)
                u, e = g.solve_confined(fullj, rc, vconf)
                print('using maximum cutoff', file=txt)
                print('rc=%.02f Bohr' % rc, file=txt)
            else:
                print('fixed energy shift', file=txt)
                print('DE=%.03f eV :: rc=%.02f Bohr'
                      % (de * Hartree, rc), file=txt)

            if vconf is not None:
                print('Potential amp=%.02f :: ri/rc=%.02f' %
                      (amplitude, ri_rel), file=txt)
            phit_g = self.smoothify(u, l)
            bf = BasisFunction(n, l, rc, phit_g,
                               '%s-sz confined orbital' % orbitaltype)
            norm = np.dot(g.dr, phit_g * phit_g)**.5
            print('Norm=%.03f' % norm, file=txt)
            singlezetas.append(bf)

            zetacounter = iter(range(2, zetacount + 1))

            if include_energy_derivatives:
                assert zetacount > 1
                zeta = next(zetacounter)
                print('\nZeta %d: %s' % (zeta, derivativedescr), file=txt)
                vconf2 = g.get_confinement_potential(amplitude,
                                                     ri_rel * rc * .99, rc)
                u2, e2 = g.solve_confined(fullj, rc, vconf2)

                phit2_g = self.smoothify(u2, l)
                dphit_g = phit2_g - phit_g
                dphit_norm = np.dot(rgd.dr_g, dphit_g * dphit_g) ** .5
                dphit_g /= dphit_norm
                descr = '%s-dz E-derivative of sz' % orbitaltype
                bf = BasisFunction(None, l, rc, dphit_g, descr)
                energy_derivative_functions.append(bf)

            for i, zeta in enumerate(zetacounter):
                print('\nZeta %d: %s' % (zeta, splitvalencedescr), file=txt)
                # Unresolved issue:  how does the lack of normalization
                # of the first function impact the tail norm scheme?
                # Presumably not much, since most interesting stuff happens
                # close to the core.
                rsplit, norm, splitwave = rsplit_by_norm(rgd, l, phit_g,
                                                         tailnorm[i]**2.0,
                                                         txt)
                descr = '%s-%sz split-valence wave' % (orbitaltype,
                                                       '0sdtq56789'[zeta])
                bf = BasisFunction(None, l, rsplit, phit_g - splitwave, descr)
                multizetas[i].append(bf)

        if polarizationcount > 0 or l_pol is not None:
            if l_pol is None:
                # Now make up some properties for the polarization orbital
                # We just use the cutoffs from the previous one times a factor
                # Find 'missing' values in lvalues
                lvalues = [l_j[vj] for vj in jvalues]
                for i in range(max(lvalues) + 1):
                    if list(lvalues).count(i) == 0:
                        l_pol = i
                        break
                else:
                    l_pol = max(lvalues) + 1

            # Find the last state with l=l_pol - 1, which will be the state we
            # base the polarization function on
            for vj, fullj, bf in zip(jvalues[::-1], fulljvalues[::-1],
                                     singlezetas[::-1]):
                if bf.l == l_pol - 1:
                    fullj_pol = fullj
                    rcut = bf.rc * rcutpol_rel
                    break
            else:
                raise ValueError('The requested value l_pol=%d requires l=%d '
                                 'among valence states' % (l_pol, l_pol - 1))
            rcut = min(rcut, rcutmax)
            msg = 'Polarization function: l=%d, rc=%.02f' % (l_pol, rcut)
            print('\n' + msg, file=txt)
            print('-' * len(msg), file=txt)
            # Make a single Gaussian for polarization function.
            #
            # It is known that for given l, the sz cutoff defined
            # by some fixed energy is strongly correlated to the
            # value of the characteristic radius which best reproduces
            # the wave function found by interpolation.
            #
            # We know that for e.g. d orbitals:
            #   rchar ~= .37 rcut[sz](.3eV)
            # Since we don't want to spend a lot of time finding
            # these value for other energies, we just find the energy
            # shift at .3 eV now

            u, e, de, vconf, rc_fixed = self.rcut_by_energy(fullj_pol,
                                                            .3, 1e-2,
                                                            6., (12., .6))

            default_rchar_rel = .25
            # Defaults for each l.  Actually we don't care right now
            rchar_rels = {}

            if rcharpol_rel is None:
                rcharpol_rel = rchar_rels.get(l_pol, default_rchar_rel)
            rchar = rcharpol_rel * rc_fixed
            gaussian = QuasiGaussian(1.0 / rchar**2, rcut)
            psi_pol = gaussian(rgd.r_g) * rgd.r_g**(l_pol + 1)
            norm = np.dot(rgd.dr_g, psi_pol * psi_pol) ** .5
            psi_pol /= norm
            print('Single quasi Gaussian', file=txt)
            msg = 'Rchar = %.03f*rcut = %.03f Bohr' % (rcharpol_rel, rchar)
            adjective = 'Gaussian'
            print(msg, file=txt)
            type = '%s-type %s polarization' % ('spdfg'[l_pol], adjective)
            bf_pol = BasisFunction(None, l_pol, rcut, psi_pol, type)

            polarization_functions.append(bf_pol)
            for i in range(polarizationcount - 1):
                npol = i + 2
                msg = '\n%s: %s' % (['Secondary', 'Tertiary', 'Quaternary',
                                     'Quintary', 'Sextary', 'Septenary'][i],
                                    splitvalencedescr)
                print(msg, file=txt)
                rsplit, norm, splitwave = rsplit_by_norm(rgd, l_pol, psi_pol,
                                                         tailnorm[i],
                                                         txt)
                descr = ('%s-type split-valence polarization %d'
                         % ('spdfg'[l_pol], npol))
                bf_pol = BasisFunction(None, l_pol, rsplit,
                                       psi_pol - splitwave,
                                       descr)
                polarization_functions.append(bf_pol)

        bf_j = []
        bf_j.extend(singlezetas)
        bf_j.extend(energy_derivative_functions)
        for multizeta_list in multizetas:
            bf_j.extend(multizeta_list)
        bf_j.extend(polarization_functions)

        rcmax = max([bf.rc for bf in bf_j])

        # The non-equidistant grids are really only suited for AE WFs
        d = 1.0 / 64
        equidistant_grid = np.arange(0.0, rcmax + d, d)
        ng = len(equidistant_grid)

        for bf in bf_j:
            # We have been storing phit_g * r, but we just want phit_g
            bf.phit_g = divrl(bf.phit_g, 1, rgd.r_g)

            gcut = min(int(1 + bf.rc / d), ng - 1)

            assert equidistant_grid[gcut] >= bf.rc
            assert equidistant_grid[gcut - 1] <= bf.rc

            bf.rc = equidistant_grid[gcut]
            # Note: bf.rc *must* correspond to a grid point (spline issues)
            bf.ng = gcut + 1
            # XXX all this should be done while building the basis vectors,
            # not here

            # Quick hack to change to equidistant coordinates
            spline = rgd.spline(bf.phit_g, rgd.r_g[rgd.floor(bf.rc)], bf.l,
                                points=100)
            bf.phit_g = np.array([spline(r) * r**bf.l
                                  for r in equidistant_grid[:bf.ng]])
            bf.phit_g[-1] = 0.

        basistype = get_basis_name(zetacount, polarizationcount)
        if self.name is None:
            compound_name = basistype
        else:
            compound_name = '%s.%s' % (self.name, basistype)

        basis = Basis(g.symbol, compound_name, False,
                      EquidistantRadialGridDescriptor(d, ng))
        basis.bf_j = bf_j
        basis.generatordata = textbuffer.getvalue().strip()
        basis.generatorattrs = {'version': version}
        textbuffer.close()

        return basis

    def grplot(self, bf_j):
        """Plot basis functions on generator's radial grid."""
        import pylab
        rc = max([bf.rc for bf in bf_j])
        g = self.generator
        r = g.r
        for bf in bf_j:
            label = bf.type
            # XXX times g.r or not times g.r ?
            pylab.plot(r, bf.phit_g / r, label=label[:12])
        axis = pylab.axis()
        newaxis = [0., rc, axis[2], axis[3]]
        pylab.axis(newaxis)
        pylab.legend()
        pylab.show()

    def plot(self, basis, figure=None, title=None, filename=None):
        """Plot basis functions using pylab."""
        # XXX method should no longer belong to a basis maker
        import pylab
        rc = max([bf.rc for bf in basis.bf_j])
        r = np.linspace(0., basis.d * (basis.ng - 1), basis.ng)
        g = self.generator
        if figure is not None:
            pylab.figure(figure)
        else:
            pylab.figure()  # not very elegant
        if title is None:
            title = g.symbol
        pylab.title(title)
        for bf in basis.bf_j:
            label = bf.type
            # XXX times g.r or not times g.r ?
            phit_g = np.zeros_like(r)
            phit_g[:len(bf.phit_g)] = bf.phit_g
            pylab.plot(r, phit_g * r, label=label[:12])
        axis = pylab.axis()
        newaxis = [0., rc, axis[2], axis[3]]
        pylab.axis(newaxis)
        pylab.legend()
        if filename is not None:
            pylab.savefig(filename)


class QuasiGaussian:
    """Gaussian-like functions for expansion of orbitals.

    Implements f(r) = A [G(r) - P(r)] where::

      G(r) = exp{- alpha r^2}
      P(r) = a - b r^2

    with (a, b) such that f(rcut) == f'(rcut) == 0.
    """
    def __init__(self, alpha, rcut, A=1.):
        self.alpha = alpha
        self.rcut = rcut
        expmar2 = np.exp(-alpha * rcut**2)
        a = (1 + alpha * rcut**2) * expmar2
        b = alpha * expmar2
        self.a = a
        self.b = b
        self.A = A

    def __call__(self, r):
        """Evaluate function values at r, which is a numpy array."""
        condition = (r < self.rcut) & (self.alpha * r**2 < 700.)
        r2 = np.where(condition, r**2., 0.)  # prevent overflow
        g = np.exp(-self.alpha * r2)
        p = (self.a - self.b * r2)
        y = np.where(condition, g - p, 0.)
        return self.A * y

    def renormalize(self, norm):
        """Divide function by norm."""
        self.A /= norm



from __future__ import print_function

from ase import Atom, Atoms
from ase.units import Bohr
from gpaw import GPAW
from gpaw.test import equal

a = 7.5 * Bohr
n = 16
atoms = Atoms([Atom('He', (0.0, 0.0, 0.0))], cell=(a, a, a), pbc=True)
calc = GPAW(gpts=(n, n, n), nbands=1, xc='LDA')
atoms.set_calculator(calc)
e1 = atoms.get_potential_energy()
niter1 = calc.get_number_of_iterations()
e1ref = calc.get_reference_energy()
de12 = calc.get_xc_difference({'name': 'PBE', 'stencil': 1})
calc.set(xc={'name': 'PBE', 'stencil': 1})
e2 = atoms.get_potential_energy()
niter2 = calc.get_number_of_iterations()
e2ref = calc.get_reference_energy()
de21 = calc.get_xc_difference('LDA')
print(e1ref + e1 + de12, e2ref + e2)
print(e1ref + e1, e2ref + e2 + de21)
print(de12, de21)
equal(e1ref + e1 + de12, e2ref + e2, 0.02)
equal(e1ref + e1, e2ref + e2 + de21, 0.025)

calc.write('PBE.gpw')

de21b = GPAW('PBE.gpw').get_xc_difference('LDA')
print(de21, de21b)
equal(de21, de21b, 9e-8)

energy_tolerance = 0.0007
niter_tolerance = 0
equal(e1, -0.0961003634812, energy_tolerance) # svnversion 5252
equal(e2, -0.0790249564625, energy_tolerance) # svnversion 5252



from gpaw import GPAW, ConvergenceError, restart
from gpaw.mixer import MixerSum
from ase.build import bulk

# bulk Fe with k-point, band, and domain parallelization
a = 2.87
atoms = bulk('Fe', 'bcc', a=a)
atoms.set_initial_magnetic_moments([2.2])
calc = GPAW(h=0.20,
            experimental={'niter_fixdensity': 2},
            eigensolver='rmm-diis',
            mixer=MixerSum(0.1, 3),
            nbands=6,
            kpts=(4, 4, 4),
            parallel={'band': 2, 'domain': (2, 1, 1)},
            maxiter=4)
atoms.set_calculator(calc)
try:
    atoms.get_potential_energy()
except ConvergenceError:
    pass
calc.write('tmp.gpw', mode='all')

# Continue calculation for few iterations
atoms, calc = restart('tmp.gpw',
                      eigensolver='rmm-diis',
                      mixer=MixerSum(0.1, 3),
                      parallel={'band': 2, 'domain': (1, 1, 2)},
                      maxiter=4)
try:
    atoms.get_potential_energy()
except ConvergenceError:
    pass



import numpy as np
from gpaw.grid_descriptor import GridDescriptor
from gpaw.test import equal

gd = GridDescriptor([4, 4, 4])
a = gd.empty(dtype=complex)
a[:] = 1.0
assert gd.integrate(a.real, a.real) == 1.0

def mic(r_v, pbc_c):
    "Mnimal image convention in an [1.0] * 3 unit cell"
    r_c = r_v % 1.0
    r_c -= (2 * r_c).astype(int)
    return np.where(pbc_c, r_c, r_v)

pbc_c = [1, 1, 0]
gd = GridDescriptor([10] * 3, cell_cv=[1.] * 3, pbc_c=pbc_c)
# Point outside box in non-periodic direction should stay
r_v = np.array([0.01, 2.49, 0.01])
dr_cG = gd.get_grid_point_distance_vectors(r_v)
equal(dr_cG[:, 0, 0, 0], mic(np.dot(gd.h_cv, gd.beg_c) - r_v, pbc_c), 1e-15)
# Point outside box in periodic direction should be folded inside
r_v = np.array([2.49, 0.01, 0.01])
dr_cG = gd.get_grid_point_distance_vectors(r_v)
equal(dr_cG[:, 0, 0, 0], mic(np.dot(gd.h_cv, gd.beg_c) - r_v, pbc_c), 1e-15)



from ase import Atoms, Atom
from gpaw import GPAW
from gpaw.atom.generator import Generator
from gpaw.atom.configurations import parameters
from gpaw import setup_paths
from gpaw.test import equal
from gpaw.mpi import world

atom = 'Ne'
setup_paths.insert(0, '.')

for xcname in ['GLLBSC','GLLB']:
    if world.rank == 0:
        g = Generator(atom, xcname =xcname, scalarrel=False,nofiles=True)
        g.run(**parameters[atom])
        eps = g.e_j[-1]
    else:
        eps = 0.0
    eps = world.sum(eps)
    world.barrier()

    a = 5
    Ne = Atoms([Atom(atom, (0, 0, 0))],
               cell=(a, a, a), pbc=False)
    Ne.center()
    calc = GPAW(nbands=7, h=0.25, xc=xcname)
    Ne.set_calculator(calc)
    e = Ne.get_potential_energy()
    # Calculate the discontinuity
    response = calc.hamiltonian.xc.xcs['RESPONSE']
    response.calculate_delta_xc()
    response.calculate_delta_xc_perturbation()

    eps3d = calc.wfs.kpt_u[0].eps_n[3]
    #if world.rank == 0:
    equal(eps, eps3d, 1e-3)
    # Correct for small cell +0.14eV (since the test needs to be fast
    # in test suite)
    equal(e+0.147106041, 0, 5e-2)



from __future__ import print_function
from ase.build import molecule
from gpaw import GPAW, PW
from gpaw.atom.derivatives import derivatives

h2 = molecule('H2O')
h2.center(vacuum=3)
h2.calc = GPAW(txt='h2o700.txt', mode=PW(700), setups='new')
print(derivatives(h2))



import numpy as np
from ase import Atoms
from gpaw import GPAW, PW, FermiDirac

ecut = 200
kpoints = [1, 1, 4]
atoms = Atoms('HLi', cell=[6, 6, 3.4], pbc=True,
              positions=[[3, 3, 0], [3, 3, 1.6]])

for xc in ['LDA', 'PBE']:
    def calculate(aug):
        atoms.calc = GPAW(mode=PW(ecut),
                          xc=xc,
                          txt='gpaw.{}.aug{}.txt'.format(xc, aug),
                          parallel={'augment_grids': aug},
                          kpts={'size': kpoints},
                          occupations=FermiDirac(width=0.1))

        def stopcalc():
            atoms.calc.scf.converged = True
        atoms.calc.attach(stopcalc, 4)

        e = atoms.get_potential_energy()
        f = atoms.get_forces()
        s = atoms.get_stress()
        return e, f, s

    e1, f1, s1 = calculate(False)
    e2, f2, s2 = calculate(True)

    eerr = abs(e2 - e1)
    ferr = np.abs(f2 - f1).max()
    serr = np.abs(s2 - s1).max()
    if atoms.calc.wfs.world.rank == 0:
        print('errs', eerr, ferr, serr)
    assert eerr < 5e-12, 'bad {} energy: err={}'.format(xc, eerr)
    assert ferr < 5e-12, 'bad {} forces: err={}'.format(xc, ferr)
    assert serr < 5e-12, 'bad {} stress: err={}'.format(xc, serr)



from __future__ import print_function
import numpy as np
from ase.build import molecule
from ase.units import Hartree
from gpaw import GPAW, Mixer
from gpaw.mpi import rank


"""
Test the dipole correction code by comparing this system:

    H
z1   O  z2
    H


(where z1 and z2 denote points where the potential is probed)

Expected potential:

      -----
     /
    /
----

to this system:

   H           H
z1  O    z2   O
   H           H


Expected potential:

       -------
      /       \
     /         \
-----           ------

The height of the two potentials are tested to be the same.

Enable if-statement in the bottom for nice plots
"""


system1 = molecule('H2O')
system1.set_pbc((True, True, False))
system1.cell = 4.0 * np.array([[1.0, -1.5, 0.0], [1.0, 1.0, 0.0],
                               [0., 0., 1.]])
system1.center(vacuum=10.0, axis=2)

system2 = system1.copy()
system2.positions *= [1.0, 1.0, -1.0]
system2 += system1
system2.center(vacuum=6.0, axis=2)

convergence = dict(density=1e-6)

def kw():
    return dict(mode='lcao',
                convergence=convergence,
                mixer=Mixer(0.5, 7, 50.0),
                h=0.25,
                xc='oldLDA')

calc1 = GPAW(poissonsolver={'dipolelayer': 'xy'}, **kw())

system1.set_calculator(calc1)
system1.get_potential_energy()
v1 = calc1.get_effective_potential(pad=False)

calc2 = GPAW(**kw())

system2.set_calculator(calc2)
system2.get_potential_energy()
v2 = calc2.get_effective_potential(pad=False)


def get_avg(v):
    nx, ny, nz = v.shape
    vyz = v.sum(axis=0) / nx
    vz = vyz.sum(axis=0) / ny
    return vz, vyz

if rank == 0:
    vz1, vyz1 = get_avg(v1)
    vz2, vyz2 = get_avg(v2)

    # Compare values that are not quite at the end of the array
    # (at the end of the array things can "oscillate" a bit)
    dvz1 = vz1[-3] - vz1[2]
    dvz2 = vz2[2] - vz2[len(vz2) // 2]
    print(dvz1, dvz2)

    err1 = abs(dvz1 - dvz2)

    correction = calc1.hamiltonian.poisson.correction

    correction_err = abs(2.0 * correction * Hartree + dvz1)
    print('correction error %s' % correction_err)
    assert correction_err < 3e-5

    # Comparison to what the values were when this test was last modified:
    ref_value = 2.07342988218
    err2 = abs(dvz1 - ref_value)

    if 0:
        import pylab as pl
        pl.imshow(vyz1)
        pl.figure()
        pl.imshow(vyz2)
        pl.figure()
        pl.plot(vz1)
        pl.plot(vz2)
        pl.show()

    print('Ref value of previous calculation', ref_value)
    print('Value in this calculation', dvz1)

    # fine grid needed to achieve convergence!
    print('Error', err1, err2)
    assert err1 < 4e-3, err1
    assert err2 < 2e-4, err2



#!/usr/bin/env python
# Copyright (C) 2003-2016  CAMP
# Please see the accompanying LICENSE file for further information.

import distutils
import distutils.util
import os
import os.path as op
import re
import subprocess
import sys
from distutils.command.build_ext import build_ext as _build_ext
from distutils.command.build_scripts import build_scripts as _build_scripts
from distutils.command.sdist import sdist as _sdist
from distutils.core import setup, Extension
from glob import glob
from pathlib import Path

from config import (get_system_config, check_dependencies,
                    write_configuration, build_interpreter, get_config_vars)


assert sys.version_info >= (3, 5)

# Get the current version number:
with open('gpaw/__init__.py', 'rb') as fd:
    txt = fd.read().decode('UTF-8')
version = re.search("__version__ = '(.*)'", txt).group(1)

description = 'GPAW: DFT and beyond within the projector-augmented wave method'
long_description = """\
GPAW is a density-functional theory (DFT) Python code based on the
projector-augmented wave (PAW) method and the atomic simulation environment
(ASE). It uses plane-waves, atom-centered basis-functions or real-space
uniform grids combined with multigrid methods."""

libraries = []
library_dirs = []
include_dirs = []
extra_link_args = []
extra_compile_args = []
runtime_library_dirs = []
extra_objects = []
define_macros = [('NPY_NO_DEPRECATED_API', 7)]
undef_macros = []

mpi_libraries = []
mpi_library_dirs = []
mpi_include_dirs = []
mpi_runtime_library_dirs = []
mpi_define_macros = []

# Search and store current git hash if possible
try:
    from ase.utils import search_current_git_hash
    githash = search_current_git_hash('gpaw')
    if githash is not None:
        define_macros += [('GPAW_GITHASH', githash)]
    else:
        print('.git directory not found. GPAW git hash not written.')
except ImportError:
    print('ASE not found. GPAW git hash not written.')

platform_id = ''

packages = []
for dirname, dirnames, filenames in os.walk('gpaw'):
    if '__init__.py' in filenames:
        packages.append(dirname.replace('/', '.'))

import_numpy = True
if '--ignore-numpy' in sys.argv:
    import_numpy = False
    sys.argv.remove('--ignore-numpy')

remove_default_flags = False
if '--remove-default-flags' in sys.argv:
    remove_default_flags = True
    sys.argv.remove('--remove-default-flags')

customize = Path('customize.py')
for i, arg in enumerate(sys.argv):
    if arg.startswith('--customize='):
        del sys.argv[i]
        customize = Path(arg.split('=')[1]).expanduser()
        break
    if arg == '--customize':
        del sys.argv[i]
        customize = Path(sys.argv.pop(i))
        break

# check for environment
# up to now LIBRARY_PATH only
try:
    for directory in os.environ['LIBRARY_PATH'].split(os.pathsep):
        if directory not in library_dirs:
            library_dirs.append(directory)
except KeyError:
    pass

get_system_config(define_macros, undef_macros,
                  include_dirs, libraries, library_dirs,
                  extra_link_args, extra_compile_args,
                  runtime_library_dirs, extra_objects,
                  import_numpy)

error = subprocess.call(['which', 'mpicc'], stdout=subprocess.PIPE)
if error:
    mpicompiler = None
else:
    mpicompiler = 'mpicc'
mpilinker = mpicompiler

compiler = None
fftw = False
scalapack = False
libvdwxc = False
elpa = False

# User provided customizations:
exec(customize.read_text())

if platform_id != '':
    my_platform = distutils.util.get_platform() + '-' + platform_id

    def my_get_platform():
        return my_platform
    distutils.util.get_platform = my_get_platform

if compiler is not None:
    # A hack to change the used compiler and linker:
    vars = get_config_vars()
    if remove_default_flags:
        for key in ['BASECFLAGS', 'CFLAGS', 'OPT', 'PY_CFLAGS',
                    'CCSHARED', 'CFLAGSFORSHARED', 'LINKFORSHARED',
                    'LIBS', 'SHLIBS']:
            if key in vars:
                value = vars[key].split()
                # remove all gcc flags (causing problems with other compilers)
                for v in list(value):
                    value.remove(v)
                vars[key] = ' '.join(value)
    for key in ['CC', 'LDSHARED']:
        if key in vars:
            value = vars[key].split()
            # first argument is the compiler/linker.  Replace with mpicompiler:
            value[0] = compiler
            vars[key] = ' '.join(value)

if scalapack:
    define_macros.append(('GPAW_WITH_SL', '1'))

if libvdwxc:
    define_macros.append(('GPAW_WITH_LIBVDWXC', '1'))

if elpa:
    define_macros.append(('GPAW_WITH_ELPA', '1'))

if fftw:
    define_macros.append(('GPAW_WITH_FFTW', '1'))

# distutils clean does not remove the _gpaw.so library and gpaw-python
# binary so do it here:
plat = distutils.util.get_platform()
plat = plat + '-' + sys.version[0:3]
gpawso = 'build/lib.%s/' % plat + '_gpaw.so'
gpawbin = 'build/bin.%s/' % plat + 'gpaw-python'
if 'clean' in sys.argv:
    if op.isfile(gpawso):
        print('removing ', gpawso)
        os.remove(gpawso)
    if op.isfile(gpawbin):
        print('removing ', gpawbin)
        os.remove(gpawbin)

sources = glob('c/*.c') + ['c/bmgs/bmgs.c']
sources = sources + glob('c/xc/*.c')
# Make build process deterministic (for "reproducible build" in debian)
sources.sort()

check_dependencies(sources)

extensions = [Extension('_gpaw',
                        sources,
                        libraries=libraries,
                        library_dirs=library_dirs,
                        include_dirs=include_dirs,
                        define_macros=define_macros,
                        undef_macros=undef_macros,
                        extra_link_args=extra_link_args,
                        extra_compile_args=extra_compile_args,
                        runtime_library_dirs=runtime_library_dirs,
                        extra_objects=extra_objects)]

files = ['gpaw-analyse-basis', 'gpaw-basis',
         'gpaw-mpisim', 'gpaw-plot-parallel-timings', 'gpaw-runscript',
         'gpaw-setup', 'gpaw-upfplot', 'gpaw']
scripts = [op.join('tools', script) for script in files]

write_configuration(define_macros, include_dirs, libraries, library_dirs,
                    extra_link_args, extra_compile_args,
                    runtime_library_dirs, extra_objects, mpicompiler,
                    mpi_libraries, mpi_library_dirs, mpi_include_dirs,
                    mpi_runtime_library_dirs, mpi_define_macros)


class sdist(_sdist):
    """Fix distutils.

    Distutils insists that there should be a README or README.txt,
    but GitLab.com needs README.rst in order to parse it as
    reStructuredText."""

    def warn(self, msg):
        if msg.startswith('standard file not found: should have one of'):
            self.filelist.append('README.rst')
        else:
            _sdist.warn(self, msg)


class build_ext(_build_ext):
    def run(self):
        _build_ext.run(self)
        if mpicompiler:
            # Also build gpaw-python:
            error = build_interpreter(
                define_macros, include_dirs, libraries,
                library_dirs, extra_link_args, extra_compile_args,
                runtime_library_dirs, extra_objects,
                mpicompiler, mpilinker, mpi_libraries,
                mpi_library_dirs,
                mpi_include_dirs,
                mpi_runtime_library_dirs, mpi_define_macros)
            assert error == 0


class build_scripts(_build_scripts):
    # Python 3's version will try to read the first line of gpaw-python
    # because it thinks it is a Python script that need an adjustment of
    # the Python version.  We skip this in our version.
    def copy_scripts(self):
        self.mkpath(self.build_dir)
        outfiles = []
        updated_files = []
        for script in self.scripts:
            outfile = op.join(self.build_dir, op.basename(script))
            outfiles.append(outfile)
            updated_files.append(outfile)
            self.copy_file(script, outfile)
        return outfiles, updated_files


if mpicompiler:
    scripts.append('build/bin.%s/' % plat + 'gpaw-python')

setup(name='gpaw',
      version=version,
      description=description,
      long_description=long_description,
      maintainer='GPAW-community',
      maintainer_email='gpaw-users@listserv.fysik.dtu.dk',
      url='https://wiki.fysik.dtu.dk/gpaw',
      license='GPLv3+',
      platforms=['unix'],
      packages=packages,
      ext_modules=extensions,
      scripts=scripts,
      cmdclass={'sdist': sdist,
                'build_ext': build_ext,
                'build_scripts': build_scripts},
      classifiers=[
          'Development Status :: 6 - Mature',
          'License :: OSI Approved :: '
          'GNU General Public License v3 or later (GPLv3+)',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Topic :: Scientific/Engineering :: Physics'])



from ase import Atoms
from gpaw import GPAW
from gpaw.lrtddft import LrTDDFT
from gpaw.lrtddft.spectrum import spectrum

txt = '-'
txt = None
load = True
load = False
xc = 'LDA'

R = 0.7  # approx. experimental bond length
a = 4.0
c = 5.0
H2 = Atoms('HH',
           [(a / 2, a / 2, (c - R) / 2),
            (a / 2, a / 2, (c + R) / 2)],
           cell=(a, a, c))

calc = GPAW(xc=xc, nbands=2, spinpol=False, eigensolver='rmm-diis', txt=txt)
H2.set_calculator(calc)
H2.get_potential_energy()
calc.write('H2saved_wfs.gpw', 'all')
calc.write('H2saved.gpw')
wfs_error = calc.wfs.eigensolver.error

print('-> starting directly after a gs calculation')
lr = LrTDDFT(calc, txt='-')
lr.diagonalize()

print('-> reading gs with wfs')
gs = GPAW('H2saved_wfs.gpw', txt=txt)

lr1 = LrTDDFT(gs, xc=xc, txt='-')
# check the oscillator strrength
assert (abs(lr1[0].get_oscillator_strength()[0] /
            lr[0].get_oscillator_strength()[0] - 1) < 1e-7)

print('-> reading gs without wfs')
gs = GPAW('H2saved.gpw', txt=None)

lr2 = LrTDDFT(gs, txt='-')
# check the oscillator strrength
d = abs(lr2[0].get_oscillator_strength()[0] /
        lr[0].get_oscillator_strength()[0] - 1)
assert (d < 2e-3), d

# write spectrum
spectrum(lr, 'spectrum.dat')



from gpaw import GPAW
from gpaw.solvation.hamiltonian import SolvationRealSpaceHamiltonian
from ase.units import Hartree, Bohr


class SolvationGPAW(GPAW):
    """Subclass of gpaw.GPAW calculator with continuum solvent model.

    See also Section III of
    A. Held and M. Walter, J. Chem. Phys. 141, 174108 (2014).
    """

    def __init__(self, cavity, dielectric, interactions=None,
                 **gpaw_kwargs):
        """Constructor for SolvationGPAW class.

        Additional arguments not present in GPAW class:
        cavity       -- A Cavity instance.
        dielectric   -- A Dielectric instance.
        interactions -- A list of Interaction instances.
        """
        if interactions is None:
            interactions = []

        self.stuff_for_hamiltonian = (cavity, dielectric, interactions)

        GPAW.__init__(self, **gpaw_kwargs)

    def create_hamiltonian(self, realspace, mode, xc):
        if not realspace:
            raise NotImplementedError(
                'SolvationGPAW does not support '
                'calculations in reciprocal space yet.')

        dens = self.density

        self.hamiltonian = SolvationRealSpaceHamiltonian(
            *self.stuff_for_hamiltonian,
            gd=dens.gd, finegd=dens.finegd,
            nspins=dens.nspins,
            collinear=dens.collinear,
            setups=dens.setups,
            timer=self.timer,
            xc=xc,
            world=self.world,
            redistributor=dens.redistributor,
            vext=self.parameters.external,
            psolver=self.parameters.poissonsolver,
            stencil=mode.interpolation)

        self.log(self.hamiltonian)

        xc.set_grid_descriptor(self.hamiltonian.finegd)

    def initialize_positions(self, atoms=None):
        spos_ac = GPAW.initialize_positions(self, atoms)
        self.hamiltonian.update_atoms(self.atoms, self.log)
        return spos_ac

    def get_electrostatic_energy(self):
        """Return electrostatic part of the total energy.

        The electrostatic part consists of everything except
        the short-range interactions defined in the interactions list.
        """
        # Energy extrapolated to zero width:
        return Hartree * self.hamiltonian.e_el_extrapolated

    def get_solvation_interaction_energy(self, subscript, atoms=None):
        """Return a specific part of the solvation interaction energy.

        The subscript parameter defines which part is to be returned.
        It has to match the value of a subscript attribute of one of
        the interactions in the interactions list.
        """
        #self.calculate(atoms, converge=True)
        return Hartree * getattr(self.hamiltonian, 'e_' + subscript)

    def get_cavity_volume(self, atoms=None):
        """Return the cavity volume in Angstrom ** 3.

        In case no volume calculator has been set for the cavity, None
        is returned.
        """
        #self.calculate(atoms, converge=True)
        V = self.hamiltonian.cavity.V
        return V and V * Bohr ** 3

    def get_cavity_surface(self, atoms=None):
        """Return the cavity surface area in Angstrom ** 2.

        In case no surface calculator has been set for the cavity,
        None is returned.
        """
        #self.calculate(atoms, converge=True)
        A = self.hamiltonian.cavity.A
        return A and A * Bohr ** 2

    def write(self, *args, **kwargs):
        raise NotImplementedError(
            'IO is not implemented yet for SolvationGPAW!')
